@@ -1,44 +1,12 @@
 /** ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code, released
- * March 31, 1998.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998-1999
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Magnus Melin <mkmelin+mozilla@iki.fi>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* This is where functions related to the standalone message window are kept */
 
 Components.utils.import("resource:///modules/jsTreeSelection.js");
+Components.utils.import("resource://gre/modules/Services.jsm");
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 Components.utils.import("resource:///modules/MsgHdrSyntheticView.js");
 
@@ -290,7 +258,7 @@ StandaloneMessageDisplayWidget.prototype = {
   onMessagesRemoved:
       function StandaloneMessageDisplayWidget_onMessagesRemoved() {
     if (this.folderDisplay.treeSelection.count == 0 &&
-        pref.getBoolPref("mail.close_message_window.on_delete")) {
+        Services.prefs.getBoolPref("mail.close_message_window.on_delete")) {
       window.close();
       return true;
     }
@@ -344,8 +312,7 @@ var messagepaneObserver = {
 function UpdateStatusMessageCounts()
 {
   // hook for extra toolbar items
-  var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
-  observerService.notifyObservers(window, "mail:updateStandAloneMessageCounts", "");
+  Services.obs.notifyObservers(window, "mail:updateStandAloneMessageCounts", "");
 }
 
 // we won't show the window until the onload() handler is finished
@@ -548,6 +515,9 @@ function ShowMenus()
   var openMail3Pane_menuitem = document.getElementById('tasksMenuMail');
   if (openMail3Pane_menuitem)
     openMail3Pane_menuitem.removeAttribute("hidden");
+  openMail3Pane_menuitem = document.getElementById('appmenu_tasksMenuMail');
+  if (openMail3Pane_menuitem)
+    openMail3Pane_menuitem.removeAttribute("hidden");
 }
 
 function HideMenus()
@@ -556,10 +526,18 @@ function HideMenus()
   if (message_menuitem)
     message_menuitem.setAttribute("hidden", "true");
 
+  message_menuitem = document.getElementById('appmenu_showMessage');
+  if (message_menuitem)
+    message_menuitem.setAttribute("hidden", "true");
+
   var folderPane_menuitem=document.getElementById('menu_showFolderPane');
   if (folderPane_menuitem)
     folderPane_menuitem.setAttribute("hidden", "true");
 
+  folderPane_menuitem = document.getElementById('appmenu_showFolderPane');
+  if (folderPane_menuitem)
+    folderPane_menuitem.setAttribute("hidden", "true");
+
   var showSearch_showMessage_Separator = document.getElementById('menu_showSearch_showMessage_Separator');
   if (showSearch_showMessage_Separator)
     showSearch_showMessage_Separator.setAttribute("hidden", "true");
@@ -572,58 +550,120 @@ function HideMenus()
   if (menuDeleteFolder)
     menuDeleteFolder.hidden = true;
 
+  menuDeleteFolder = document.getElementById('appmenu_deleteFolder');
+  if (menuDeleteFolder)
+    menuDeleteFolder.hidden = true;
+
   var renameFolderMenu = document.getElementById('menu_renameFolder');
   if (renameFolderMenu)
     renameFolderMenu.setAttribute("hidden", "true");
 
+  renameFolderMenu = document.getElementById('appmenu_renameFolder');
+  if (renameFolderMenu)
+    renameFolderMenu.setAttribute("hidden", "true");
+
   var viewLayoutMenu = document.getElementById("menu_MessagePaneLayout");
   if (viewLayoutMenu)
     viewLayoutMenu.setAttribute("hidden", "true");
 
+  viewLayoutMenu = document.getElementById("appmenu_MessagePaneLayout");
+  if (viewLayoutMenu)
+    viewLayoutMenu.setAttribute("hidden", "true");
+
+  let paneViewSeparator = document.getElementById("appmenu_paneViewSeparator");
+  if (paneViewSeparator)
+    paneViewSeparator.setAttribute("hidden", "true");
+
   var viewFolderMenu = document.getElementById("menu_FolderViews");
   if (viewFolderMenu)
     viewFolderMenu.setAttribute("hidden", "true");
 
+  viewFolderMenu = document.getElementById("appmenu_FolderViews");
+  if (viewFolderMenu)
+    viewFolderMenu.setAttribute("hidden", "true");
+
   var viewMessagesMenu = document.getElementById('viewMessagesMenu');
   if (viewMessagesMenu)
     viewMessagesMenu.setAttribute("hidden", "true");
 
+  viewMessagesMenu = document.getElementById('appmenu_viewMessagesMenu');
+  if (viewMessagesMenu)
+    viewMessagesMenu.setAttribute("hidden", "true");
+
   var viewMessageViewMenu = document.getElementById('viewMessageViewMenu');
   if (viewMessageViewMenu)
     viewMessageViewMenu.setAttribute("hidden", "true");
 
+  viewMessageViewMenu = document.getElementById('appmenu_viewMessageViewMenu');
+  if (viewMessageViewMenu)
+    viewMessageViewMenu.setAttribute("hidden", "true");
+
   var viewMessagesMenuSeparator = document.getElementById('viewMessagesMenuSeparator');
   if (viewMessagesMenuSeparator)
     viewMessagesMenuSeparator.setAttribute("hidden", "true");
 
+  viewMessagesMenuSeparator = document.getElementById('appmenu_viewMessagesMenuSeparator');
+  if (viewMessagesMenuSeparator)
+    viewMessagesMenuSeparator.setAttribute("hidden", "true");
+
   var openMessageMenu = document.getElementById('openMessageWindowMenuitem');
   if (openMessageMenu)
     openMessageMenu.setAttribute("hidden", "true");
 
+  openMessageMenu = document.getElementById('appmenu_openMessageWindowMenuitem');
+  if (openMessageMenu)
+    openMessageMenu.setAttribute("hidden", "true");
+
   var viewSortMenuSeparator = document.getElementById('viewSortMenuSeparator');
   if (viewSortMenuSeparator)
     viewSortMenuSeparator.setAttribute("hidden", "true");
 
+  viewSortMenuSeparator = document.getElementById('appmenu_viewAfterThreadsSeparator');
+  if (viewSortMenuSeparator)
+    viewSortMenuSeparator.setAttribute("hidden", "true");
+
   var viewSortMenu = document.getElementById('viewSortMenu');
   if (viewSortMenu)
     viewSortMenu.setAttribute("hidden", "true");
 
+  viewSortMenu = document.getElementById('appmenu_viewSortMenu');
+  if (viewSortMenu)
+    viewSortMenu.setAttribute("hidden", "true");
+
   var emptryTrashMenu = document.getElementById('menu_emptyTrash');
   if (emptryTrashMenu)
     emptryTrashMenu.setAttribute("hidden", "true");
 
+  emptryTrashMenu = document.getElementById('appmenu_emptyTrash');
+  if (emptryTrashMenu)
+    emptryTrashMenu.setAttribute("hidden", "true");
+
   var menuPropertiesSeparator = document.getElementById("editPropertiesSeparator");
   if (menuPropertiesSeparator)
     menuPropertiesSeparator.setAttribute("hidden", "true");
 
+  menuPropertiesSeparator = document.getElementById("appmenu_editPropertiesSeparator");
+  if (menuPropertiesSeparator)
+    menuPropertiesSeparator.setAttribute("hidden", "true");
+
   var menuProperties = document.getElementById('menu_properties');
   if (menuProperties)
     menuProperties.setAttribute("hidden", "true");
 
+  menuProperties = document.getElementById('appmenu_properties');
+  if (menuProperties)
+    menuProperties.setAttribute("hidden", "true");
+
   var favoriteFolder = document.getElementById('menu_favoriteFolder');
   if (favoriteFolder)
   {
-    favoriteFolder.disabled = true;
+    favoriteFolder.setAttribute("disabled", "true");
+    favoriteFolder.setAttribute("hidden", "true");
+  }
+
+  favoriteFolder = document.getElementById('appmenu_favoriteFolder');
+  if (favoriteFolder) {
+    favoriteFolder.setAttribute("disabled", "true");
     favoriteFolder.setAttribute("hidden", "true");
   }
 
@@ -631,18 +671,50 @@ function HideMenus()
   if (compactFolderMenu)
     compactFolderMenu.setAttribute("hidden", "true");
 
+  compactFolderMenu = document.getElementById('appmenu_compactFolder');
+  if (compactFolderMenu)
+    compactFolderMenu.setAttribute("hidden", "true");
+
   var trashSeparator = document.getElementById('trashMenuSeparator');
   if (trashSeparator)
     trashSeparator.setAttribute("hidden", "true");
 
+  let fileMenuAfterRenameSeparator = document.getElementById('appmenu_fileMenuAfterRenameSeparator');
+  if (fileMenuAfterRenameSeparator)
+    fileMenuAfterRenameSeparator.setAttribute("hidden", "true");
+
+  let fileMenuAfterCompactSeparator = document.getElementById('appmenu_fileMenuAfterCompactSeparator');
+  if (fileMenuAfterCompactSeparator)
+    fileMenuAfterCompactSeparator.setAttribute("hidden", "true");
+
   var goStartPageSeparator = document.getElementById('goNextSeparator');
   if (goStartPageSeparator)
     goStartPageSeparator.hidden = true;
 
+  goStartPageSeparator = document.getElementById('appmenu_goNextSeparator');
+  if (goStartPageSeparator)
+    goStartPageSeparator.hidden = true;
+
+  let goRecentlyClosedTabsSeparator = document.getElementById('goRecentlyClosedTabsSeparator');
+  if (goRecentlyClosedTabsSeparator)
+    goRecentlyClosedTabsSeparator.setAttribute("hidden", "true");
+
+  goRecentlyClosedTabsSeparator = document.getElementById('appmenu_goRecentlyClosedTabsSeparator');
+  if (goRecentlyClosedTabsSeparator)
+    goRecentlyClosedTabsSeparator.setAttribute("hidden", "true");
+
   var goStartPage = document.getElementById('goStartPage');
   if (goStartPage)
    goStartPage.hidden = true;
 
+  goStartPage = document.getElementById('appmenu_goStartPage');
+  if (goStartPage)
+   goStartPage.hidden = true;
+
+  let quickFilterBar = document.getElementById('appmenu_quickFilterBar');
+  if (quickFilterBar)
+   quickFilterBar.hidden = true;
+
   var menuFileClose = document.getElementById('menu_close');
   var menuFileQuit = document.getElementById('menu_FileQuitItem');
   if (menuFileClose && menuFileQuit)
@@ -688,7 +760,7 @@ function ReloadMessage()
 // MessageWindowController object (handles commands when one of the trees does not have focus)
 var MessageWindowController =
 {
-   supportsCommand: function(command)
+  supportsCommand: function(command)
   {
     switch ( command )
     {
@@ -702,6 +774,8 @@ var MessageWindowController =
       case "cmd_shiftDelete":
       case "button_shiftDelete":
       case "cmd_tag":
+      case "cmd_addTag":
+      case "cmd_manageTags":
       case "cmd_removeTags":
       case "cmd_tag1":
       case "cmd_tag2":
@@ -751,7 +825,6 @@ var MessageWindowController =
       case "button_goForward":
       case "button_goBack":
         return gFolderDisplay.selectedMessage != null;
-
       case "cmd_reply":
       case "button_reply":
       case "cmd_replySender":
@@ -791,6 +864,7 @@ var MessageWindowController =
       case "cmd_viewAllHeader":
       case "cmd_viewNormalHeader":
       case "cmd_stop":
+      case "cmd_chat":
         return true;
       case "cmd_synchronizeOffline":
       case "cmd_downloadFlagged":
@@ -857,6 +931,8 @@ var MessageWindowController =
       case "cmd_reload":
       case "cmd_find":
       case "cmd_tag":
+      case "cmd_addTag":
+      case "cmd_manageTags":
       case "cmd_removeTags":
       case "cmd_tag1":
       case "cmd_tag2":
@@ -935,10 +1011,10 @@ var MessageWindowController =
         return SetupUndoRedoCommand(command);
       case "cmd_moveToFolderAgain":
         loadedFolder = gFolderDisplay.displayedFolder;
-        if (!loadedFolder || (pref.getBoolPref("mail.last_msg_movecopy_was_move") &&
+        if (!loadedFolder || (Services.prefs.getBoolPref("mail.last_msg_movecopy_was_move") &&
             !loadedFolder.canDeleteMessages))
           return false;
-        let targetURI = pref.getCharPref("mail.last_msg_movecopy_target_uri");
+        let targetURI = Services.prefs.getCharPref("mail.last_msg_movecopy_target_uri");
         if (!targetURI)
           return false;
         let targetFolder = MailUtils.getFolderForURI(targetURI);
@@ -948,6 +1024,8 @@ var MessageWindowController =
       case "cmd_runJunkControls":
       case "cmd_deleteJunk":
         return false;
+      case "cmd_chat":
+        return true;
       default:
         return false;
     }
@@ -1009,8 +1087,8 @@ var MessageWindowController =
         MsgEditMessageAsNew();
         break;
       case "cmd_moveToFolderAgain":
-        var folderId = pref.getCharPref("mail.last_msg_movecopy_target_uri");
-        if (pref.getBoolPref("mail.last_msg_movecopy_was_move"))
+        var folderId = Services.prefs.getCharPref("mail.last_msg_movecopy_target_uri");
+        if (Services.prefs.getBoolPref("mail.last_msg_movecopy_was_move"))
           MsgMoveMessage(GetMsgFolderFromUri(folderId));
         else
           MsgCopyMessage(GetMsgFolderFromUri(folderId));
@@ -1064,6 +1142,12 @@ var MessageWindowController =
       case "cmd_search":
         MsgSearchMessages();
         break;
+      case "cmd_addTag":
+        AddTag();
+        return;
+      case "cmd_manageTags":
+        ManageTags();
+        return;
       case "cmd_removeTags":
         RemoveAllMessageTags();
         return;
@@ -1182,6 +1266,18 @@ var MessageWindowController =
       case "cmd_stop":
         msgWindow.StopUrls();
         break;
+      case "cmd_chat":
+        let win = Services.wm.getMostRecentWindow("mail:3pane");
+        if (win) {
+          win.focus();
+          win.showChatTab();
+        }
+        else {
+          window.openDialog("chrome://messenger/content/", "_blank",
+                            "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar",
+                            null, {tabType: "chat", tabParams: {}});
+        }
+        break;
       }
   },
 
