@@ -1,39 +1,6 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Communicator client code, released
- * March 31, 1998.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1998-1999
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
 Components.utils.import("resource://gre/modules/Services.jsm");
@@ -218,22 +185,18 @@ const DOMLinkHandler = {
       // Verify that the load of this icon is legal.
       // Some error or special pages can load their favicon.
       // To be on the safe side, only allow chrome:// favicons.
-      let isAllowedPage = [
-        /^about:neterror\?/,
-        /^about:blocked\?/,
-        /^about:certerror\?/,
-        /^about:home$/
-      ].some(function (re) { re.test(targetDoc.documentURI); });
+      let isAllowedPage = (targetDoc.documentURI == "about:home") ||
+        ["about:neterror?",
+         "about:blocked?",
+         "about:certerror?"
+        ].some(function (aStart) { targetDoc.documentURI.startsWith(aStart); });
 
       if (!isAllowedPage || !uri.schemeIs("chrome")) {
         // Be extra paraniod and just make sure we're not going to load
         // something we shouldn't. Firefox does this, so we're doing the same.
-        let ssm = Components.classes["@mozilla.org/scriptsecuritymanager;1"]
-          .getService(Components.interfaces.nsIScriptSecurityManager);
-
           try {
-            ssm.checkLoadURIWithPrincipal(targetDoc.nodePrincipal, uri,
-                                          Components.interfaces.nsIScriptSecurityManager.DISALLOW_SCRIPT);
+            Services.scriptSecurityManager.checkLoadURIWithPrincipal(targetDoc.nodePrincipal, uri,
+                                           Components.interfaces.nsIScriptSecurityManager.DISALLOW_SCRIPT);
           }
           catch (ex) {
             return;
@@ -523,15 +486,12 @@ var specialTabs = {
 
     let tabmail = document.getElementById('tabmail');
 
-    var prefs = Components.classes["@mozilla.org/preferences-service;1"]
-                          .getService(Components.interfaces.nsIPrefBranch);
-
     tabmail.registerTabType(this.contentTabType);
     tabmail.registerTabType(this.chromeTabType);
 
     // If we've upgraded (note: always get these values so that we set
     // the mstone preference for the new version):
-    let [fromVer, toVer] = this.getApplicationUpgradeVersions(prefs);
+    let [fromVer, toVer] = this.getApplicationUpgradeVersions();
 
     // Although this might not be really necessary because of the version checks, we'll
     // check this pref anyway and clear it so that we are consistent with what Firefox
@@ -549,10 +509,10 @@ var specialTabs = {
     }
 
     // Show the about rights notification if we need to.
-    if (this.shouldShowAboutRightsNotification(prefs))
-      this.showAboutRightsNotification(prefs);
-    else if (this.shouldShowTelemetryNotification(prefs))
-      this.showTelemetryNotification(prefs);
+    if (this.shouldShowAboutRightsNotification())
+      this.showAboutRightsNotification();
+    else if (this.shouldShowTelemetryNotification())
+      this.showTelemetryNotification();
   },
 
   /**
@@ -600,12 +560,16 @@ var specialTabs = {
       clone.setAttribute("id", "contentTab" + this.lastBrowserId);
       clone.setAttribute("collapsed", false);
 
+      let toolbox = clone.firstChild;
+      toolbox.setAttribute("id", "contentTabToolbox" + this.lastBrowserId);
+      toolbox.firstChild.setAttribute("id", "contentTabToolbar" + this.lastBrowserId);
+
       aTab.panel.appendChild(clone);
       aTab.root = clone;
 
       // Start setting up the browser.
-      aTab.browser = aTab.panel.getElementsByTagName("browser")[0];
-      aTab.toolbar = aTab.panel.getElementsByClassName("contentTabToolbar")[0];
+      aTab.browser = aTab.panel.querySelector("browser");
+      aTab.toolbar = aTab.panel.querySelector(".contentTabToolbar");
 
       // As we're opening this tab, showTab may not get called, so set
       // the type according to if we're opening in background or not.
@@ -628,7 +592,7 @@ var specialTabs = {
       gPluginHandler.addEventListeners(aTab.browser);
 
       // Now initialise the find bar.
-      aTab.findbar = aTab.panel.getElementsByTagName("findbar")[0];
+      aTab.findbar = aTab.panel.querySelector("findbar");
       aTab.findbar.setAttribute("browserid",
                                 "contentTabBrowser" + this.lastBrowserId);
 
@@ -723,12 +687,12 @@ var specialTabs = {
    * or the pref being set to ignore - return null and the current version.
    * In either case, updates the pref with the latest version.
    */
-  getApplicationUpgradeVersions: function(prefs) {
+  getApplicationUpgradeVersions: function() {
     let savedAppVersion = null;
     let prefstring = "mailnews.start_page_override.mstone";
 
     try {
-      savedAppVersion = prefs.getCharPref(prefstring);
+      savedAppVersion = Services.prefs.getCharPref(prefstring);
     } catch (ex) {}
 
     let currentApplicationVersion = Application.version;
@@ -737,7 +701,7 @@ var specialTabs = {
       return [null, this.splitVersion(currentApplicationVersion)];
 
     if (savedAppVersion != currentApplicationVersion)
-      prefs.setCharPref(prefstring, currentApplicationVersion);
+      Services.prefs.setCharPref(prefstring, currentApplicationVersion);
 
     return [this.splitVersion(savedAppVersion), this.splitVersion(currentApplicationVersion)];
   },
@@ -771,16 +735,31 @@ var specialTabs = {
    *
    * This is controlled by the pref toolkit.telemetry.prompted
    */
-  shouldShowTelemetryNotification: function(prefs) {
-    // toolkit has decided that the pref should have no default value
+  shouldShowTelemetryNotification: function() {
+    // For a transitional period (expected end in mid-2013) we allow the pref
+    // kTelemetryPrompted to be bool or int and reset it to bool.
+    // See bug 807848 for more details.
     try {
-      if (prefs.getBoolPref(kTelemetryPrompted) || prefs.getBoolPref(kTelemetryEnabled))
+      if (Services.prefs.getIntPref(kTelemetryPrompted) > 0) {
+        Services.prefs.clearUserPref(kTelemetryPrompted);
+        Services.prefs.setBoolPref(kTelemetryPrompted, true);
+        return false;
+      }
+    } catch (e) {
+      try {
+        if (Services.prefs.getBoolPref(kTelemetryPrompted))
+          return false;
+      } catch (e) { }
+    }
+    try {
+      // toolkit has decided that the pref should have no default value
+      if (Services.prefs.getBoolPref(kTelemetryEnabled))
         return false;
     } catch (e) { }
     return true;
   },
 
-  showTelemetryNotification: function(prefs) {
+  showTelemetryNotification: function() {
     var notifyBox = document.getElementById("mail-notification-box");
 
     var brandBundle =
@@ -789,7 +768,7 @@ var specialTabs = {
       new StringBundle("chrome://messenger/locale/telemetry.properties");
 
     var productName = brandBundle.get("brandFullName");
-    var serverOwner = prefs.getCharPref(kTelemetryServerOwner);
+    var serverOwner = Services.prefs.getCharPref(kTelemetryServerOwner);
     var telemetryText = telemetryBundle.get("telemetryText", [productName, serverOwner]);
 
     var buttons = [
@@ -798,7 +777,7 @@ var specialTabs = {
         accessKey: telemetryBundle.get("telemetryYesButtonAccessKey"),
         popup:     null,
         callback:  function(aNotificationBar, aButton) {
-          prefs.setBoolPref(kTelemetryEnabled, true);
+          Services.prefs.setBoolPref(kTelemetryEnabled, true);
         }
       },
       {
@@ -810,7 +789,7 @@ var specialTabs = {
     ];
 
     // Set pref to indicate we've shown the notification.
-    prefs.setBoolPref(kTelemetryPrompted, true);
+    Services.prefs.setBoolPref(kTelemetryPrompted, true);
 
     var notification = notifyBox.appendNotification(telemetryText, "telemetry", null, notifyBox.PRIORITY_INFO_LOW, buttons);
     notification.persistence = 3; // arbitrary number, just so bar sticks around for a bit
@@ -847,24 +826,21 @@ var specialTabs = {
    *     If this pref isn't set or the value is less than the current version
    *     then we show the about:rights notification.
    */
-  shouldShowAboutRightsNotification: function(prefs) {
+  shouldShowAboutRightsNotification: function() {
     try {
-      return !prefs.getBoolPref("mail.rights.override");
+      return !Services.prefs.getBoolPref("mail.rights.override");
     } catch (e) { }
 
-    return prefs.getIntPref("mail.rights.version") < this._kAboutRightsVersion;
+    return Services.prefs.getIntPref("mail.rights.version") < this._kAboutRightsVersion;
   },
 
-  showAboutRightsNotification: function(prefs) {
+  showAboutRightsNotification: function() {
     var notifyBox = document.getElementById("mail-notification-box");
 
-    var stringBundle =
-      Components.classes["@mozilla.org/intl/stringbundle;1"]
-                .getService(Components.interfaces.nsIStringBundleService);
     var brandBundle =
-      stringBundle.createBundle("chrome://branding/locale/brand.properties");
+      Services.strings.createBundle("chrome://branding/locale/brand.properties");
     var rightsBundle =
-      stringBundle.createBundle("chrome://messenger/locale/aboutRights.properties");
+      Services.strings.createBundle("chrome://messenger/locale/aboutRights.properties");
 
     var productName = brandBundle.GetStringFromName("brandFullName");
     var notifyRightsText = rightsBundle.formatStringFromName("notifyRightsText",
@@ -891,7 +867,7 @@ var specialTabs = {
     box.persistence = 3;
 
     // Set the pref to say we've displayed the notification.
-    prefs.setIntPref("mail.rights.version", this._kAboutRightsVersion);
+    Services.prefs.setIntPref("mail.rights.version", this._kAboutRightsVersion);
   },
 
   /**
@@ -914,6 +890,7 @@ var specialTabs = {
         openLinkExternally(href);
       }
     }
+    return false;
   },
 
   /**
@@ -941,6 +918,7 @@ var specialTabs = {
         openLinkExternally(href);
       }
     }
+    return false;
   },
 
   /**
@@ -978,6 +956,7 @@ var specialTabs = {
         openLinkExternally(href);
       }
     }
+    return false;
   },
 
   chromeTabType: {
@@ -1024,10 +1003,14 @@ var specialTabs = {
       clone.setAttribute("id", "chromeTab" + this.lastBrowserId);
       clone.setAttribute("collapsed", false);
 
+      let toolbox = clone.firstChild;
+      toolbox.setAttribute("id", "chromeTabToolbox" + this.lastBrowserId);
+      toolbox.firstChild.setAttribute("id", "chromeTabToolbar" + this.lastBrowserId);
+
       aTab.panel.appendChild(clone);
 
       // Start setting up the browser.
-      aTab.browser = aTab.panel.getElementsByTagName("browser")[0];
+      aTab.browser = aTab.panel.querySelector("browser");
 
       // As we're opening this tab, showTab may not get called, so set
       // the type according to if we're opening in background or not.
@@ -1444,8 +1427,9 @@ var specialTabs = {
    */
   setTabIcon: function(aTab, aIcon) {
     if (aIcon && this.mFaviconService)
-      this.mFaviconService.setAndFetchFaviconForPage(aTab.browser.currentURI,
-                                                     makeURI(aIcon), false);
+      this.mFaviconService.setAndFetchFaviconForPage(
+        aTab.browser.currentURI, makeURI(aIcon), false,
+        this.mFaviconService.FAVICON_LOAD_NON_PRIVATE);
 
     // Save this off so we know about it later,
     aTab.browser.mIconURL = aIcon;
