@@ -1,39 +1,6 @@
-/* ***** BEGIN LICENSE BLOCK *****
- *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Thunderbird Mail Client.
- *
- * The Initial Developer of the Original Code is
- * the Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2009
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Andrew Sutherland <asutherland@asutherland.org>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 const EXPORTED_SYMBOLS = ['DBViewWrapper', 'IDBViewWrapperListener'];
 
@@ -42,6 +9,7 @@ const Ci = Components.interfaces;
 const Cr = Components.results;
 const Cu = Components.utils;
 
+Cu.import("resource:///modules/mailServices.js");
 Cu.import("resource:///modules/mailViewManager.js");
 Cu.import("resource:///modules/searchSpec.js");
 Cu.import("resource:///modules/virtualFolderWrapper.js");
@@ -93,18 +61,12 @@ var FolderNotificationHelper = {
    */
   _init: function FolderNotificationHelper__init() {
     // register with the session for our folded loaded notifications
-    let mailSession =
-      Cc["@mozilla.org/messenger/services/session;1"]
-        .getService(Ci.nsIMsgMailSession);
-    mailSession.AddFolderListener(this,
-                                  Ci.nsIFolderListener.event |
-                                  Ci.nsIFolderListener.intPropertyChanged);
+    MailServices.mailSession.AddFolderListener(this,
+                                               Ci.nsIFolderListener.event |
+                                               Ci.nsIFolderListener.intPropertyChanged);
 
     // register with the notification service for deleted folder notifications
-    let notificationService =
-      Cc["@mozilla.org/messenger/msgnotificationservice;1"]
-        .getService(Ci.nsIMsgFolderNotificationService);
-    notificationService.addListener(this,
+    MailServices.mfn.addListener(this,
       Ci.nsIMsgFolderNotificationService.folderDeleted |
       // we need to track renames because we key off of URIs. frick.
       Ci.nsIMsgFolderNotificationService.folderRenamed |
@@ -1008,7 +970,7 @@ DBViewWrapper.prototype = {
       if (mailViewTag && mailViewTag != "0" && mailViewTag != "1") {
         // the tag gets stored with a ":" on the front, presumably done
         //  as a means of name-spacing that was never subsequently leveraged.
-        if (mailViewTag[0] == ":")
+        if (mailViewTag.startsWith(":"))
           mailViewTag = mailViewTag.substr(1);
         // (the true is so we don't persist)
         this.setMailView(MailViewConstants.kViewItemTags, mailViewTag, true);
@@ -1175,7 +1137,7 @@ DBViewWrapper.prototype = {
     if (aOldFolder == this.displayedFolder)
       this.displayedFolder = aNewFolder;
 
-    // indexOf does't work for this (reliably)
+    // indexOf doesn't work for this (reliably)
     for each (let [i,underlyingFolder] in Iterator(this._underlyingFolders)) {
       if (aOldFolder == underlyingFolder) {
         this._underlyingFolders[i] = aNewFolder;
@@ -1211,7 +1173,7 @@ DBViewWrapper.prototype = {
       return;
     }
 
-    // indexOf does't work for this (reliably)
+    // indexOf doesn't work for this (reliably)
     for each (let [i,underlyingFolder] in Iterator(this._underlyingFolders)) {
       if (aFolder == underlyingFolder) {
         this._underlyingFolders.splice(i,1);
