@@ -6,6 +6,7 @@
 var gAdvancedPane = {
   mPane: null,
   mInitialized: false,
+  mShellServiceWorking: false,
 
   init: function ()
   {
@@ -53,6 +54,21 @@ var gAdvancedPane = {
       document.getElementById("searchintegration.enable").disabled = true;
     }
 
+#ifdef HAVE_SHELL_SERVICE
+    // If the shell service is not working, disable the "Check now" button
+    // and "perform check at startup" checkbox.
+    try {
+      let shellSvc = Components.classes["@mozilla.org/mail/shell-service;1"]
+                               .getService(Components.interfaces.nsIShellService);
+      this.mShellServiceWorking = true;
+    } catch (ex) {
+      document.getElementById("alwaysCheckDefault").disabled = true;
+      document.getElementById("alwaysCheckDefault").checked = false;
+      document.getElementById("checkDefaultButton").disabled = true;
+      this.mShellServiceWorking = false;
+    }
+#endif
+
     this.mInitialized = true;
   },
 
@@ -75,17 +91,13 @@ var gAdvancedPane = {
    */
   checkDefaultNow: function (aAppType)
   {
-    var nsIShellService = Components.interfaces.nsIShellService;
-    var shellSvc;
-    try {
-      shellSvc = Components.classes["@mozilla.org/mail/shell-service;1"]
-                           .getService(nsIShellService);
-    } catch (ex) { return; }
+    if (!this.mShellServiceWorking)
+      return;
 
     // otherwise, bring up the default client dialog
     window.openDialog("chrome://messenger/content/systemIntegrationDialog.xul",
                       "SystemIntegration",
-                      "modal,centerscreen,chrome,resizable=no");
+                      "modal,centerscreen,chrome,resizable=no", "calledFromPrefs");
   },
 #endif
 
