@@ -28,12 +28,30 @@ sys.path.append(SCRIPT_DIRECTORY)
 from automation import Automation
 automation = Automation()
 
-from automationutils import checkForCrashes
+# --------------------------------------------------------------
+# TODO: this is a hack for mozbase without virtualenv, remove with bug 849900
+#
+here = os.path.dirname(__file__)
+mozbase = os.path.realpath(os.path.join(os.path.dirname(here), 'mozbase'))
+
+try:
+    import mozcrash
+except:
+    deps = ['mozcrash',
+            'mozlog']
+    for dep in deps:
+        module = os.path.join(mozbase, dep)
+        if module not in sys.path:
+            sys.path.append(module)
+    import mozcrash
+# ---------------------------------------------------------------
+
 from time import sleep
 import imp
 
 PROFILE_DIR = os.path.join(SCRIPT_DIRECTORY, 'mozmillprofile')
 SYMBOLS_PATH = None
+PLUGINS_PATH = None
 # XXX This breaks any semblance of test runner modularity, and only works
 # because we know that we run MozMill only once per process. This needs to be
 # fixed if that ever changes.
@@ -103,7 +121,7 @@ class ThunderTestProfile(mozrunner.ThunderbirdProfile):
         # Do not allow check new mail to be set
         'mail.startup.enabledMailCheckOnce' :  True,
         # Disable compatibility checking
-        'extensions.checkCompatibility.19.0': False,
+        'extensions.checkCompatibility.22.0': False,
         # Stop any pings to AMO on add-on install
         'extensions.getAddons.cache.enabled': False,
         # Disable test pilot new tab (this can be set to anything currently, just needs to be set).
@@ -141,13 +159,13 @@ class ThunderTestProfile(mozrunner.ThunderbirdProfile):
         'mail.identity.id1.htmlSigFormat' : False,
         'mail.identity.id1.htmlSigText' : "Tinderbox is soo 90ies",
         'mail.identity.id1.smtpServer' :  "smtp1",
-        'mail.identity.id1.useremail' :  "tinderbox@invalid.com",
+        'mail.identity.id1.useremail' :  "tinderbox@foo.invalid",
         'mail.identity.id1.valid' :  True,
         'mail.identity.id2.fullName' : "Tinderboxpushlog",
         'mail.identity.id2.htmlSigFormat' : True,
         'mail.identity.id2.htmlSigText' : "Tinderboxpushlog is the new <b>hotness!</b>",
         'mail.identity.id2.smtpServer' : "smtp1",
-        'mail.identity.id2.useremail' : "tinderboxpushlog@invalid.com",
+        'mail.identity.id2.useremail' : "tinderboxpushlog@foo.invalid",
         'mail.identity.id2.valid' : True,
         'mail.server.server1.directory-rel' :  "[ProfD]Mail/Local Folders",
         'mail.server.server1.hostname' :  "Local Folders",
@@ -159,21 +177,21 @@ class ThunderTestProfile(mozrunner.ThunderbirdProfile):
         'mail.server.server2.download_on_biff' :  True,
         'mail.server.server2.hostname' :  "tinderbox",
         'mail.server.server2.login_at_startup' :  False,
-        'mail.server.server2.name' :  "tinderbox@invalid.com",
+        'mail.server.server2.name' :  "tinderbox@foo.invalid",
         'mail.server.server2.type' :  "pop3",
         'mail.server.server2.userName' :  "tinderbox",
         'mail.server.server2.whiteListAbURI': "",
         'mail.server.server3.hostname' :  "prpl-irc",
         'mail.server.server3.imAccount' :  "account1",
         'mail.server.server3.type' :  "im",
-        'mail.server.server3.userName' :  "mozmilltest@irc.mozilla.org",
+        'mail.server.server3.userName' :  "mozmilltest@irc.mozilla.invalid",
         'mail.smtp.defaultserver' :  "smtp1",
         'mail.smtpserver.smtp1.hostname' :  "tinderbox",
         'mail.smtpserver.smtp1.username' :  "tinderbox",
         'mail.smtpservers' :  "smtp1",
         'messenger.account.account1.autoLogin' :  False,
         'messenger.account.account1.firstConnectionState' :  1,
-        'messenger.account.account1.name' :  "mozmilltest@irc.mozilla.org",
+        'messenger.account.account1.name' :  "mozmilltest@irc.mozilla.invalid",
         'messenger.account.account1.prpl' :  "prpl-irc",
         'messenger.accounts' :  "account1",
     }
@@ -192,6 +210,13 @@ class ThunderTestProfile(mozrunner.ThunderbirdProfile):
         if not os.path.exists(PROFILE_DIR):
             raise Exception('somehow failed to create profile dir!')
 
+        if PLUGINS_PATH:
+          if not os.path.exists(PLUGINS_PATH):
+            raise Exception('Plugins path "%s" does not exist.' % PLUGINS_PATH)
+
+          dest = os.path.join(PROFILE_DIR, "plugins")
+          shutil.copytree(PLUGINS_PATH, dest)
+
         if wrapper is not None and hasattr(wrapper, "on_profile_created"):
             # It's a little dangerous to allow on_profile_created access to the
             # profile object, because it isn't fully initalized yet
@@ -315,9 +340,11 @@ class ThunderTestCLI(mozmill.CLI):
     parser_options = copy.copy(mozmill.CLI.parser_options)
     parser_options[('--symbols-path',)] = {"default": None, "dest": "symbols",
                                            "help": "The path to the symbol files from build_symbols"}
+    parser_options[('--plugins-path',)] = {"default": None, "dest": "plugins",
+                                           "help": "The path to the plugins directory for the created profile"}
 
     def __init__(self, *args, **kwargs):
-        global SYMBOLS_PATH, TEST_NAME
+        global SYMBOLS_PATH, PLUGINS_PATH, TEST_NAME
 
         # mozmill 1.5.4 still explicitly hardcodes references to Firefox; in
         # order to avoid missing out on initializer logic or needing to copy
@@ -331,6 +358,7 @@ class ThunderTestCLI(mozmill.CLI):
         mozmill.CLI.__init__(self, *args, **kwargs)
 
         SYMBOLS_PATH = self.options.symbols
+        PLUGINS_PATH = self.options.plugins
         if isinstance(self.options.test, basestring):
             test_paths = [self.options.test]
         else:
@@ -475,8 +503,8 @@ def dumpRichResults():
         print '##### MOZMILL-RICH-FAILURES-END #####'
 
 def checkCrashesAtExit():
-    if checkForCrashes(os.path.join(PROFILE_DIR, 'minidumps'), SYMBOLS_PATH,
-                       TEST_NAME):
+    if mozcrash.check_for_crashes(os.path.join(PROFILE_DIR, 'minidumps'), SYMBOLS_PATH,
+                                  TEST_NAME):
         print >> sys.stderr, 'TinderboxPrint: ' + TEST_NAME + '<br/><em class="testfail">CRASH</em>'
         sys.exit(1)
 
