@@ -28,8 +28,9 @@
 Components.utils.import("resource:///modules/iteratorUtils.jsm");
 Components.utils.import("resource://gre/modules/Services.jsm");
 Components.utils.import("resource:///modules/mailServices.js");
+Components.utils.import("resource:///modules/folderUtils.jsm");
+Components.utils.import("resource:///modules/hostnameUtils.jsm");
 
-var gSmtpHostNameIsIllegal = false;
 // If Local directory has changed the app needs to restart. Once this is set
 // a restart will be attempted at each attempt to close the Account manager with OK.
 var gRestartNeeded = false;
@@ -46,6 +47,42 @@ var currentPageId;
 var pendingAccount;
 var pendingPageId;
 
+/**
+ * This array contains filesystem folders that are deemed inappropriate
+ * for use as the local directory pref for message storage.
+ * It is global to allow extensions to add to/remove from it if needed.
+ * Extentions adding new server types should first consider setting
+ * nsIMsgProtocolInfo(of the server type).defaultLocalPath properly
+ * so that the test will allow that directory automatically.
+ * See the checkLocalDirectoryIsSafe function for description of the members.
+ */
+var gDangerousLocalStorageDirs = [
+  // profile folder
+  { dirsvc: "ProfD",    OS: null },
+  // GRE install folder
+  { dirsvc: "GreD",     OS: null },
+  // Application install folder
+  { dirsvc: "CurProcD", OS: null },
+  // system temporary folder
+  { dirsvc: "TmpD",     OS: null },
+  // Windows system folder
+  { dirsvc: "SysD",     OS: "WINNT" },
+  // Windows folder
+  { dirsvc: "WinD",     OS: "WINNT" },
+  // Program Files folder
+  { dirsvc: "ProgF",    OS: "WINNT" },
+  // trash folder
+  { dirsvc: "Trsh",     OS: "Darwin" },
+  // Mac OS system folder
+  { dir:    "/System",  OS: "Darwin" },
+  // devices folder
+  { dir:    "/dev",     OS: "Darwin,Linux" },
+  // process info folder
+  { dir:    "/proc",    OS: "Linux" },
+  // system state folder
+  { dir:    "/sys",     OS: "Linux" }
+];
+
 // This sets an attribute in a xul element so that we can later
 // know what value to substitute in a prefstring.  Different
 // preference types set different attributes.  We get the value
@@ -72,15 +109,15 @@ function updateElementWithKeys(account, element, type) {
 }
 
 function hideShowControls(serverType) {
-  var controls = document.getElementsByAttribute("hidefor", "*");
-  for (var controlNo = 0; controlNo < controls.length; controlNo++) {
-    var control = controls[controlNo];
-    var hideFor = control.getAttribute("hidefor");
+  let controls = document.querySelectorAll("[hidefor]");
+  for (let controlNo = 0; controlNo < controls.length; controlNo++) {
+    let control = controls[controlNo];
+    let hideFor = control.getAttribute("hidefor");
 
     // Hide unsupported server types using hideFor="servertype1,servertype2".
-    var hide = false;
-    var hideForTokens = hideFor.split(",");
-    for (var tokenNo = 0; tokenNo < hideForTokens.length; tokenNo++) {
+    let hide = false;
+    let hideForTokens = hideFor.split(",");
+    for (let tokenNo = 0; tokenNo < hideForTokens.length; tokenNo++) {
       if (hideForTokens[tokenNo] == serverType) {
         hide = true;
         break;
@@ -133,6 +170,8 @@ function selectServer(server, selectPageId)
       let account = childrenNode.childNodes[i]._account;
       if (account && server == account.incomingServer) {
         accountNode = childrenNode.childNodes[i];
+        // Make sure all the panes of the account to be selected are shown.
+        accountNode.setAttribute("open", "true");
         break;
       }
     }
@@ -146,8 +185,7 @@ function selectServer(server, selectPageId)
     let pageId = accountNode.getAttribute("PageTag");
     if (pageId != selectPageId) {
       // ... or one of its children.
-      let pages = accountNode.getElementsByAttribute("PageTag", selectPageId);
-      pageToSelect = pages[0];
+      pageToSelect = accountNode.querySelector('[PageTag="' + selectPageId + '"]');
     }
   }
 
@@ -205,11 +243,6 @@ function onAccept(aDoChecks) {
     // Check if user/host have been modified correctly.
     if (!checkUserServerChanges(true))
       return false;
-
-    if (gSmtpHostNameIsIllegal) {
-      gSmtpHostNameIsIllegal = false;
-      return false;
-    }
   }
 
   if (!onSave())
@@ -262,6 +295,112 @@ function checkDirectoryIsValid(aLocalPath) {
   return true;
 }
 
+/**
+ * Even if the local path is usable, there are some special folders we do not
+ * want to allow for message storage as they cause problems (see e.g. bug 750781).
+ *
+ * aLocalPath  The nsIFile of a directory to check.
+ */
+function checkDirectoryIsAllowed(aLocalPath) {
+  /**
+   * Check if the local path (aLocalPath) is 'safe' i.e. NOT a parent
+   * or subdirectory of the given special system/app directory (aDirToCheck).
+   *
+   * @param aDirToCheck  An object describing the special directory.
+   *        The object has the following members:
+   *        dirsvc      : A path keyword to retrieve from the Directory service.
+   *        dir         : An absolute filesystem path.
+   *                      Only one of 'dirsvc' or 'dir' can be specified.
+   *        OS          : A string of comma separated values defining on which
+   *                      Operating systems the folder is unusable:
+   *                       null   = all
+   *                       WINNT  = Windows
+   *                       Darwin = OS X
+   *                       Linux  = Linux
+   *        safeSubdirs : An array of directory names that are allowed to be used
+   *                      under the tested directory.
+   * @param aLocalPath  An nsIFile of the directory to check, intended for message storage.
+   */
+  function checkLocalDirectoryIsSafe(aDirToCheck, aLocalPath) {
+    if (aDirToCheck.OS) {
+      if (aDirToCheck.OS.split(",").indexOf(Services.appinfo.OS) == -1)
+        return true;
+    }
+
+    let testDir = null;
+    if ("dirsvc" in aDirToCheck) {
+      try {
+        testDir = Services.dirsvc.get(aDirToCheck.dirsvc, Components.interfaces.nsIFile);
+      } catch (e) {
+        Components.utils.reportError("The special folder " + aDirToCheck.dirsvc +
+          " cannot be retrieved on this platform: " + e);
+      }
+
+      if (!testDir)
+        return true;
+    }
+    else if ("dir" in aDirToCheck) {
+      testDir = Components.classes["@mozilla.org/file/local;1"]
+                          .createInstance(Components.interfaces.nsIFile);
+      testDir.initWithPath(aDirToCheck.dir);
+      if (!testDir.exists())
+        return true;
+    } else {
+      Components.utils.reportError("No directory to check?");
+      return true;
+    }
+
+    testDir.normalize();
+
+    if (testDir.equals(aLocalPath) || aLocalPath.contains(testDir, true))
+      return false;
+
+    if (testDir.contains(aLocalPath, true)) {
+      if (!("safeSubdirs" in aDirToCheck))
+        return false;
+
+      // While the tested directory may not be safe,
+      // a subdirectory of some safe subdirectories may be fine.
+      let isInSubdir = false;
+      for (let subDir of aDirToCheck.safeSubdirs) {
+        let checkDir = testDir.clone();
+        checkDir.append(subDir);
+        if (checkDir.contains(aLocalPath, true)) {
+          isInSubdir = true;
+          break;
+        }
+      }
+      return isInSubdir;
+    }
+
+    return true;
+  } // end of checkDirectoryIsNotSpecial
+
+  // If the server type has a nsIMsgProtocolInfo.defaultLocalPath set,
+  // allow that directory.
+  if (currentAccount.incomingServer) {
+    try {
+      let defaultPath = Components
+        .classes["@mozilla.org/messenger/protocol/info;1?type=" +
+                 currentAccount.incomingServer.type]
+        .getService(Components.interfaces.nsIMsgProtocolInfo)
+        .defaultLocalPath;
+      if (defaultPath) {
+        defaultPath.normalize();
+        if (defaultPath.contains(aLocalPath, true))
+          return true;
+      }
+    } catch (e) { /* No problem if this fails. */ }
+  }
+
+  for (let tryDir of gDangerousLocalStorageDirs) {
+    if (!checkLocalDirectoryIsSafe(tryDir, aLocalPath))
+      return false;
+  }
+
+  return true;
+}
+
 /**
  * Check if the specified directory does meet all the requirements
  * for safe mail storage.
@@ -271,6 +410,8 @@ function checkDirectoryIsValid(aLocalPath) {
 function checkDirectoryIsUsable(aLocalPath) {
   const kAlertTitle = document.getElementById("bundle_prefs")
                               .getString("prefPanel-server");
+  const originalPath = aLocalPath;
+
   let invalidPath = false;
   try{
     aLocalPath.normalize();
@@ -278,16 +419,25 @@ function checkDirectoryIsUsable(aLocalPath) {
 
   if (invalidPath || !checkDirectoryIsValid(aLocalPath)) {
     let alertString = document.getElementById("bundle_prefs")
-                              .getString("localDirectoryInvalid");
+                              .getFormattedString("localDirectoryInvalid",
+                                                  [originalPath.path]);
     Services.prompt.alert(window, kAlertTitle, alertString);
     return false;
   }
 
+  if (!checkDirectoryIsAllowed(aLocalPath)) {
+    let alertNotAllowed = document.getElementById("bundle_prefs")
+                                  .getFormattedString("localDirectoryNotAllowed",
+                                                      [originalPath.path]);
+    Services.prompt.alert(window, kAlertTitle, alertNotAllowed);
+    return false;
+  }
+
   // Check that no other account has this same or dependent local directory.
   let allServers = MailServices.accounts.allServers;
 
-  for each (let server in fixIterator(allServers,
-                                      Components.interfaces.nsIMsgIncomingServer))
+  for (let server in fixIterator(allServers,
+                                 Components.interfaces.nsIMsgIncomingServer))
   {
     if (server.key == currentAccount.incomingServer.key)
       continue;
@@ -332,18 +482,6 @@ function checkUserServerChanges(showAlert) {
   const prefBundle = document.getElementById("bundle_prefs");
   const alertTitle = prefBundle.getString("prefPanel-server");
   var alertText = null;
-  if (MailServices.smtp.defaultServer) {
-    try {
-      var smtpHostName = top.frames["contentFrame"]
-                            .document.getElementById("smtp.hostname");
-      if (smtpHostName && hostnameIsIllegal(smtpHostName.value)) {
-        alertText = prefBundle.getString("enterValidHostname");
-        Services.prompt.alert(window, alertTitle, alertText);
-        gSmtpHostNameIsIllegal = true;
-      }
-    }
-    catch (ex) {}
-  }
 
   var accountValues = getValueArrayFor(currentAccount);
   if (!accountValues)
@@ -356,12 +494,12 @@ function checkUserServerChanges(showAlert) {
   // Get the new username, hostname and type from the page
   var newUser, newHost, newType, oldUser, oldHost;
   var uIndx, hIndx;
-  for (var i = 0; i < pageElements.length; i++) {
+  for (let i = 0; i < pageElements.length; i++) {
     if (pageElements[i].id) {
-      var vals = pageElements[i].id.split(".");
+      let vals = pageElements[i].id.split(".");
       if (vals.length >= 2) {
-        var type = vals[0];
-        var slot = vals[1];
+        let type = vals[0];
+        let slot = pageElements[i].id.slice(type.length + 1);
 
         // if this type doesn't exist (just removed) then return.
         if (!(type in accountValues) || !accountValues[type]) return true;
@@ -391,14 +529,26 @@ function checkUserServerChanges(showAlert) {
   alertText = null;
   // If something is changed then check if the new user/host already exists.
   if ((oldUser != newUser) || (oldHost != newHost)) {
-    if ((checkUser && (newUser == "")) || (newHost == "")) {
-        alertText = prefBundle.getString("serverNameEmpty");
-    } else {
-      let sameServer = MailServices.accounts
-                                   .findRealServer(newUser, newHost, newType, 0)
-      if (sameServer && (sameServer != currentAccount.incomingServer))
-        alertText = prefBundle.getString("modifiedAccountExists");
+    newUser = newUser.trim();
+    newHost = cleanUpHostName(newHost);
+    if (checkUser && (newUser == "")) {
+      alertText = prefBundle.getString("userNameEmpty");
     }
+    else if (!isLegalHostNameOrIP(newHost)) {
+      alertText = prefBundle.getString("enterValidServerName");
+    }
+    else {
+      let sameServer = MailServices.accounts
+                                   .findRealServer(newUser, newHost, newType, 0);
+      if (sameServer && (sameServer != currentAccount.incomingServer)) {
+        alertText = prefBundle.getString("modifiedAccountExists");
+      } else {
+        // New hostname passed all checks. We may have cleaned it up so set
+        // the new value back into the input element.
+        setFormElementValue(pageElements[hIndx], newHost);
+      }
+    }
+
     if (alertText) {
       if (showAlert)
         Services.prompt.alert(window, alertTitle, alertText);
@@ -516,8 +666,10 @@ function AddIMAccount()
  * optionally un-highlight the previous one.
  */
 function markDefaultServer(newDefault, oldDefault) {
-  let accountTree = document.getElementById("account-tree-children");
-  for each (let accountNode in accountTree.childNodes) {
+  let accountTreeNodes = document.getElementById("account-tree-children")
+                                 .childNodes;
+  for (let i = 0; i < accountTreeNodes.length; i++) {
+    let accountNode = accountTreeNodes[i];
     if (newDefault == accountNode._account) {
       accountNode.firstChild
                  .firstChild
@@ -776,7 +928,7 @@ function updateItems(tree, account, addAccountItem, setDefaultItem, removeItem)
     let type = server.type;
 
     if (account != MailServices.accounts.defaultAccount &&
-        server.canBeDefaultServer && account.identities.Count() > 0)
+        server.canBeDefaultServer && account.identities.length > 0)
       canSetDefault = true;
 
     if (Components.classes["@mozilla.org/messenger/protocol/info;1?type=" + type]
@@ -855,12 +1007,6 @@ function onAccountTreeSelect(pageId, account)
       pageId = currentPageId;
     }
 
-    if (gSmtpHostNameIsIllegal) {
-      gSmtpHostNameIsIllegal = false;
-      selectServer(currentAccount.incomingServer, currentPageId);
-      return true;
-    }
-
     if (gRestartNeeded)
       onAccept(false);
   }
@@ -924,7 +1070,7 @@ function onPanelLoaded(pageId) {
   pendingPageId = null;
 }
 
-function loadPage(pageId)
+function pageURL(pageId)
 {
   let chromePackageName;
   try {
@@ -937,8 +1083,14 @@ function loadPage(pageId)
   catch (ex) {
     chromePackageName = "messenger";
   }
+  return "chrome://" + chromePackageName + "/content/" + pageId;
+}
+
+function loadPage(pageId)
+{
   const LOAD_FLAGS_NONE = Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE;
-  document.getElementById("contentFrame").webNavigation.loadURI("chrome://" + chromePackageName + "/content/" + pageId, LOAD_FLAGS_NONE, null, null, null);
+  document.getElementById("contentFrame").webNavigation.loadURI(pageURL(pageId),
+    LOAD_FLAGS_NONE, null, null, null);
 }
 
 // save the values of the widgets to the given server
@@ -960,12 +1112,12 @@ function savePage(account)
     return;
 
   // store the value in the account
-  for (var i = 0; i < pageElements.length; i++) {
+  for (let i = 0; i < pageElements.length; i++) {
     if (pageElements[i].id) {
-      var vals = pageElements[i].id.split(".");
+      let vals = pageElements[i].id.split(".");
       if (vals.length >= 2) {
-        var type = vals[0];
-        var slot = vals[1];
+        let type = vals[0];
+        let slot = pageElements[i].id.slice(type.length + 1);
 
         setAccountValue(accountValues,
                         type, slot,
@@ -1069,12 +1221,13 @@ function restorePage(pageId, account)
     return;
 
   // restore the value from the account
-  for (var i = 0; i < pageElements.length; i++) {
+  for (let i = 0; i < pageElements.length; i++) {
     if (pageElements[i].id) {
-      var vals = pageElements[i].id.split(".");
+      let vals = pageElements[i].id.split(".");
       if (vals.length >= 2) {
-        var type = vals[0];
-        var slot = vals[1];
+        let type = vals[0];
+        let slot = pageElements[i].id.slice(type.length + 1);
+
         // buttons are lockable, but don't have any data so we skip that part.
         // elements that do have data, we get the values at poke them in.
         if (pageElements[i].localName != "button") {
@@ -1144,9 +1297,6 @@ function getFormElementValue(formElement) {
 
 // sets the value of a widget
 function setFormElementValue(formElement, value) {
-
-  //formElement.value = formElement.defaultValue;
-  //  formElement.checked = formElement.defaultChecked;
   var type = formElement.localName;
   if (type == "checkbox") {
     if (value == undefined) {
@@ -1161,20 +1311,11 @@ function setFormElementValue(formElement, value) {
         formElement.checked = value;
     }
   }
-
   else if (type == "radiogroup" || type =="menulist") {
-
-    var selectedItem;
-    if (value == undefined) {
-      if (type == "radiogroup")
-        selectedItem = formElement.firstChild;
-      else
-        selectedItem = formElement.firstChild.firstChild;
-    }
+    if (value == undefined)
+      formElement.selectedIndex = 0;
     else
-      selectedItem = formElement.getElementsByAttribute("value", value)[0];
-
-    formElement.selectedItem = selectedItem;
+      formElement.value = value;
   }
   // handle nsILocalFile
   else if (type == "textbox" &&
@@ -1262,6 +1403,36 @@ var gAccountTree = {
   },
   onServerChanged: function at_onServerChanged(aServer) {},
 
+  _rdf: Components.classes["@mozilla.org/rdf/rdf-service;1"]
+                  .getService(Components.interfaces.nsIRDFService),
+  _rdfDataSource: null,
+  _rdfOpenAttribute: null,
+
+  /**
+   * Retrieve from localstore.rdf whether the account should be expanded (open)
+   * in the account tree.
+   *
+   * @param aAccountKey  key of the account to check
+   */
+  _getAccountOpenState: function at_getAccountOpenState(aAccountKey) {
+    // The code for this was ported from
+    // mozilla/browser/components/nsBrowserGlue.js.
+    if (!this._rdfDataSource) {
+      this._rdfDataSource = this._rdf.GetDataSource("rdf:local-store");
+      this._rdfOpenAttribute = this._rdf.GetResource("open");
+    }
+
+    // Retrieve the persisted value from localstore.rdf.
+    // It is stored under the URI of the current document and ID of the XUL element.
+    let resource = this._rdf.GetResource(document.documentURI + "#" + aAccountKey);
+    let target = this._rdfDataSource.GetTarget(resource, this._rdfOpenAttribute, true);
+    if (target instanceof Components.interfaces.nsIRDFLiteral)
+      return target.Value;
+
+    // If there was no value stored, use opened state.
+    return "true";
+  },
+
   _build: function at_build() {
     const Ci = Components.interfaces;
     var bundle = document.getElementById("bundle_prefs");
@@ -1273,36 +1444,9 @@ var gAccountTree = {
                   {string: get("prefPanel-addressing"), src: "am-addressing.xul"},
                   {string: get("prefPanel-junk"), src: "am-junk.xul"}];
 
-    // Get our account list, and add the proper items
-    var mgr = MailServices.accounts;
+    let accounts = allAccountsSorted(false);
 
-    var accounts = [a for each (a in fixIterator(mgr.accounts, Ci.nsIMsgAccount))];
-    // Stupid bug 41133 hack. Grr...
-    accounts = accounts.filter(function fix(a) { return a.incomingServer; });
-
-    function sortAccounts(a, b) {
-      if (a.key == mgr.defaultAccount.key)
-        return -1;
-      if (b.key == mgr.defaultAccount.key)
-        return 1;
-      var aIsNews = a.incomingServer.type == "nntp";
-      var bIsNews = b.incomingServer.type == "nntp";
-      if (aIsNews && !bIsNews)
-        return 1;
-      if (bIsNews && !aIsNews)
-        return -1;
-
-      var aIsLocal = a.incomingServer.type == "none";
-      var bIsLocal = b.incomingServer.type == "none";
-      if (aIsLocal && !bIsLocal)
-        return 1;
-      if (bIsLocal && !aIsLocal)
-        return -1;
-      return 0;
-    }
-    accounts.sort(sortAccounts);
-
-    var mainTree = document.getElementById("account-tree-children");
+    let mainTree = document.getElementById("account-tree-children");
     // Clear off all children...
     while (mainTree.firstChild)
       mainTree.removeChild(mainTree.firstChild);
@@ -1324,8 +1468,8 @@ var gAccountTree = {
 
       // Now add our panels
       var panelsToKeep = [];
-      var idents = mgr.GetIdentitiesForServer(server);
-      if (idents.Count()) {
+      let idents = MailServices.accounts.getIdentitiesForServer(server);
+      if (idents.length) {
         panelsToKeep.push(panels[0]); // The server panel is valid
         panelsToKeep.push(panels[1]); // also the copies panel
         panelsToKeep.push(panels[4]); // and addresssing
@@ -1379,14 +1523,19 @@ var gAccountTree = {
           kidtreeitem._account = account;
         }
         treeitem.setAttribute("container", "true");
-        treeitem.setAttribute("open", "true");
+        treeitem.id = account.key;
+        // Load the 'open' state of the account from localstore.rdf.
+        treeitem.setAttribute("open", this._getAccountOpenState(account.key));
+        // Let the localstore.rdf automatically save the 'open' state of the
+        // account when it is changed.
+        treeitem.setAttribute("persist", "open");
       }
       treeitem.setAttribute("PageTag", server ? server.accountManagerChrome
                                               : "am-main.xul");
       treeitem._account = account;
     }
 
-    markDefaultServer(mgr.defaultAccount, null);
+    markDefaultServer(MailServices.accounts.defaultAccount, null);
 
     // Now add the outgoing server node
     var treeitem = document.createElement("treeitem");
