@@ -1,41 +1,9 @@
 /* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * David Ascher <davida@mozilla.com> and
- * Ben Bucksch <ben.bucksch beonex.com>
- * Portions created by the Initial Developer are Copyright (C) 2008-2011
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either of the GNU General Public License Version 2 or later (the "GPL"),
- * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+Components.utils.import("resource:///modules/mailServices.js");
 Components.utils.import("resource://gre/modules/Services.jsm");
 
 /**
@@ -64,7 +32,6 @@ Components.utils.import("resource://gre/modules/Services.jsm");
 
 // from http://xyfer.blogspot.com/2005/01/javascript-regexp-email-validator.html
 var emailRE = /^[-_a-z0-9\'+*$^&%=~!?{}]+(?:\.[-_a-z0-9\'+*$^&%=~!?{}]+)*@(?:[-a-z0-9.]+\.[a-z]{2,6}|\d{1,3}(?:\.\d{1,3}){3})(?::\d+)?$/i;
-var domainRE = /^((?:[\w-]+\.)*\w[\w-]{0,66})\.([a-z]{2,6}(?:\.[a-z]{2})?)$|(\[?(\d{1,3}\.){3}\d{1,3}\]?)$/i
 const kHighestPort = 65535;
 
 Cu.import("resource:///modules/gloda/log4moz.js");
@@ -176,9 +143,20 @@ EmailConfigWizard.prototype =
      * account or to show it to the user), you need to run replaceVariables().
      */
     this._currentConfig = null;
+
+    let userFullname;
+    try {
+      let userInfo = Cc["@mozilla.org/userinfo;1"].getService(Ci.nsIUserInfo);
+      userFullname = userInfo.fullname;
+    } catch(e) {
+      // nsIUserInfo may not be implemented on all platforms, and name might
+      // not be avaialble even if it is.
+    }
+
     this._domain = "";
     this._email = "";
-    this._realname = "";
+    this._realname = (userFullname) ? userFullname : "";
+    e("realname").value = this._realname;
     this._password = "";
     this._okCallback = null;
 
@@ -191,6 +169,8 @@ EmailConfigWizard.prototype =
       }
     }
 
+    gEmailWizardLogger.info("Email account setup dialog loaded.");
+
     gStringsBundle = e("strings");
     gMessengerBundle = e("bundle_messenger");
     gBrandShortName = e("bundle_brand").getString("brandShortName");
@@ -220,15 +200,13 @@ EmailConfigWizard.prototype =
     // Populate SMTP server dropdown with already configured SMTP servers from
     // other accounts.
     var menulist = e("outgoing_hostname");
-    var smtpManager = Cc["@mozilla.org/messengercompose/smtp;1"]
-        .getService(Ci.nsISmtpService);
-    var smtpServers = smtpManager.smtpServers;
+    let smtpServers = MailServices.smtp.servers;
     while (smtpServers.hasMoreElements()) {
       let server = smtpServers.getNext().QueryInterface(Ci.nsISmtpServer);
       let label = server.displayname;
       let key = server.key;
-      if (smtpManager.defaultServer &&
-          smtpManager.defaultServer.key == key) {
+      if (MailServices.smtp.defaultServer &&
+          MailServices.smtp.defaultServer.key == key) {
         label += " " + gStringsBundle.getString("default_server_tag");
       }
       let menuitem = menulist.appendItem(label, key, ""); // label,value,descr
@@ -239,7 +217,7 @@ EmailConfigWizard.prototype =
     menuitem.serverKey = null;
 
     // admin-locked prefs hurray
-    if (!Application.prefs.getValue("signon.rememberSignons", true)) {
+    if (!Services.prefs.getBoolPref("signon.rememberSignons")) {
       let rememberPasswordE = e("remember_password");
       rememberPasswordE.checked = false;
       rememberPasswordE.disabled = true;
@@ -692,7 +670,7 @@ EmailConfigWizard.prototype =
     if (!this._realname || !this._email) {
       return;
     }
-    return this._foundConfig2(config);
+    this._foundConfig2(config);
   },
 
   // Continuation of foundConfig2() after custom fields.
@@ -932,8 +910,8 @@ EmailConfigWizard.prototype =
     }
     config.incoming.type = sanitize.translate(e("incoming_protocol").value,
         { 1: "imap", 2 : "pop3", 0 : null });
-    config.incoming.socketType = parseInt(e("incoming_ssl").value);
-    config.incoming.auth = parseInt(e("incoming_authMethod").value);
+    config.incoming.socketType = sanitize.integer(e("incoming_ssl").value);
+    config.incoming.auth = sanitize.integer(e("incoming_authMethod").value);
     config.incoming.username = e("incoming_username").value;
 
     // Outgoing server
@@ -963,8 +941,8 @@ EmailConfigWizard.prototype =
       } catch (e) {
         config.outgoing.port = undefined; // incl. default "Auto"
       }
-      config.outgoing.socketType = e("outgoing_ssl").value;
-      config.outgoing.auth = e("outgoing_authMethod").value;
+      config.outgoing.socketType = sanitize.integer(e("outgoing_ssl").value);
+      config.outgoing.auth = sanitize.integer(e("outgoing_authMethod").value);
       config.outgoing.username = config.incoming.username;
     }
 
@@ -1357,7 +1335,7 @@ EmailConfigWizard.prototype =
   onAdvancedSetup : function()
   {
     assert(this._currentConfig instanceof AccountConfig);
-    var configFilledIn = this.getConcreteConfig();
+    let configFilledIn = this.getConcreteConfig();
 
     if (checkIncomingServerAlreadyExists(configFilledIn)) {
       alertPrompt(gStringsBundle.getString("error_creating_account"),
@@ -1366,11 +1344,9 @@ EmailConfigWizard.prototype =
     }
 
     gEmailWizardLogger.info("creating account in backend");
-    var newAccount = createAccountInBackend(configFilledIn);
+    let newAccount = createAccountInBackend(configFilledIn);
 
-    var windowManager = Cc["@mozilla.org/appshell/window-mediator;1"]
-        .getService(Ci.nsIWindowMediator);
-    var existingAccountManager = windowManager
+    let existingAccountManager = Services.wm
         .getMostRecentWindow("mailnews:accountmanager");
     if (existingAccountManager) {
       existingAccountManager.focus();
@@ -1578,6 +1554,8 @@ EmailConfigWizard.prototype =
         // should back-port it to the current config.
         self._currentConfig.incoming.auth = successfulConfig.incoming.auth;
         self._currentConfig.outgoing.auth = successfulConfig.outgoing.auth;
+        self._currentConfig.incoming.username = successfulConfig.incoming.username;
+        self._currentConfig.outgoing.username = successfulConfig.outgoing.username;
         self.finish();
       },
       function(e) // failed
@@ -1604,8 +1582,6 @@ EmailConfigWizard.prototype =
 };
 
 var gEmailConfigWizard = new EmailConfigWizard();
-gEmailWizardLogger.info("email account setup dialog");
-
 
 function serverMatches(a, b)
 {
