@@ -5,6 +5,7 @@
 
 #include "nscore.h"
 #include "nsMsgUtils.h"
+#include "nsArrayUtils.h"
 #include "nsMessengerOSXIntegration.h"
 #include "nsIMsgMailSession.h"
 #include "nsIMsgIncomingServer.h"
@@ -32,7 +33,6 @@
 #include "nsIAlertsService.h"
 #include "nsIStringBundle.h"
 #include "nsToolkitCompsCID.h"
-#include "nsINotificationsList.h"
 #include "nsIMsgDatabase.h"
 #include "nsIMsgHdr.h"
 #include "nsIMsgHeaderParser.h"
@@ -183,11 +183,9 @@ NS_INTERFACE_MAP_END
 nsresult
 nsMessengerOSXIntegration::Init()
 {
-  // need to register a named Growl notification
   nsresult rv;
   nsCOMPtr<nsIObserverService> observerService = do_GetService("@mozilla.org/observer-service;1", &rv);
   NS_ENSURE_SUCCESS(rv, rv);
-  observerService->AddObserver(this, "before-growl-registration", false);
   return observerService->AddObserver(this, "mail-startup-done", false);
 }
 
@@ -253,28 +251,6 @@ nsMessengerOSXIntegration::Observe(nsISupports* aSubject, const char* aTopic, co
     return mailSession->AddFolderListener(this, nsIFolderListener::boolPropertyChanged | nsIFolderListener::intPropertyChanged);
   }
 
-  // register named Growl notification for new mail alerts.
-  if (!strcmp(aTopic, "before-growl-registration"))
-  {
-    nsresult rv;
-    nsCOMPtr<nsIObserverService> observerService = do_GetService("@mozilla.org/observer-service;1", &rv);
-    if (NS_SUCCEEDED(rv))
-      observerService->RemoveObserver(this, "before-growl-registration");
-
-    nsCOMPtr<nsINotificationsList> notifications = do_QueryInterface(aSubject, &rv);
-    if (NS_SUCCEEDED(rv))
-    {
-      nsCOMPtr<nsIStringBundle> bundle;
-      GetStringBundle(getter_AddRefs(bundle));
-      if (bundle)
-      {
-        nsString growlNotification;
-        bundle->GetStringFromName(NS_LITERAL_STRING("growlNotification").get(), getter_Copies(growlNotification));
-        notifications->AddNotification(growlNotification, true);
-      }
-    }
-  }
-
   if (!strcmp(aTopic, kNewChatMessageTopic)) {
     // We don't have to bother about checking if the window is already focused
     // before attempting to bounce the dock icon, as BounceDockIcon is
@@ -407,25 +383,16 @@ nsMessengerOSXIntegration::ShowAlertMessage(const nsAString& aAlertTitle,
 
   if (showAlert)
   {
-    // Use growl if installed
     nsCOMPtr<nsIAlertsService> alertsService (do_GetService(NS_ALERTSERVICE_CONTRACTID, &rv));
+    // If we have an nsIAlertsService implementation, use it:
     if (NS_SUCCEEDED(rv))
     {
-      nsCOMPtr<nsIStringBundle> bundle;
-      GetStringBundle(getter_AddRefs(bundle));
-      if (bundle)
-      {
-        nsString growlNotification;
-        bundle->GetStringFromName(NS_LITERAL_STRING("growlNotification").get(),
-                                  getter_Copies(growlNotification));
-        rv = alertsService->ShowAlertNotification(NS_LITERAL_STRING(kNewMailAlertIcon),
-                                                  aAlertTitle,
-                                                  aAlertText,
-                                                  true,
-                                                  NS_ConvertASCIItoUTF16(aFolderURI),
-                                                  this,
-                                                  growlNotification);
-      }
+      alertsService->ShowAlertNotification(NS_LITERAL_STRING(kNewMailAlertIcon),
+                                           aAlertTitle, aAlertText, true,
+                                           NS_ConvertASCIItoUTF16(aFolderURI),
+                                           this, EmptyString(),
+                                           NS_LITERAL_STRING("auto"),
+                                           EmptyString());
     }
 
     BounceDockIcon();
@@ -678,7 +645,7 @@ nsMessengerOSXIntegration::GetNewMailAuthors(nsIMsgFolder* aFolder,
 
           notify->SetData(true);
           os->NotifyObservers(notify, "newmail-notification-requested",
-                              PromiseFlatString(author).get());
+                              author.get());
 
           bool includeSender;
           notify->GetData(&includeSender);
@@ -709,22 +676,21 @@ nsMessengerOSXIntegration::GetFirstFolderWithNewMail(nsIMsgFolder* aFolder, nsCS
   {
     nsCOMPtr<nsIMsgFolder> msgFolder;
     // enumerate over the folders under this root folder till we find one with new mail....
-    nsCOMPtr<nsISupportsArray> allFolders;
-    NS_NewISupportsArray(getter_AddRefs(allFolders));
-    nsresult rv = aFolder->ListDescendents(allFolders);
+    nsCOMPtr<nsIArray> allFolders;
+    nsresult rv = aFolder->GetDescendants(getter_AddRefs(allFolders));
     NS_ENSURE_SUCCESS(rv, rv);
 
-    nsCOMPtr<nsIEnumerator> enumerator;
-    allFolders->Enumerate(getter_AddRefs(enumerator));
-    if (enumerator)
+    nsCOMPtr<nsISimpleEnumerator> enumerator;
+    rv = allFolders->Enumerate(getter_AddRefs(enumerator));
+    if (NS_SUCCEEDED(rv) && enumerator)
     {
       nsCOMPtr<nsISupports> supports;
       int32_t numNewMessages = 0;
-      nsresult more = enumerator->First();
-      while (NS_SUCCEEDED(more))
+      bool hasMore = false;
+      while (NS_SUCCEEDED(enumerator->HasMoreElements(&hasMore)) && hasMore)
       {
-        rv = enumerator->CurrentItem(getter_AddRefs(supports));
-        if (supports)
+        rv = enumerator->GetNext(getter_AddRefs(supports));
+        if (NS_SUCCEEDED(rv) && supports)
         {
           msgFolder = do_QueryInterface(supports, &rv);
           if (msgFolder)
@@ -733,7 +699,6 @@ nsMessengerOSXIntegration::GetFirstFolderWithNewMail(nsIMsgFolder* aFolder, nsCS
             msgFolder->GetNumNewMessages(false, &numNewMessages);
             if (numNewMessages)
               break; // kick out of the while loop
-            more = enumerator->Next();
           }
         } // if we have a folder
       }  // if we have more potential folders to enumerate
