@@ -47,6 +47,8 @@
 #include "nsAppDirectoryServiceDefs.h"
 #include "mozilla/Services.h"
 #include "nsIMsgFilter.h"
+#include "nsIArray.h"
+#include "nsArrayUtils.h"
 
 #define PORT_NOT_SET -1
 
@@ -2120,42 +2122,38 @@ nsMsgIncomingServer::GetSpamFilterPlugin(nsIMsgFilterPlugin **aFilterPlugin)
 
 // get all the servers that defer to the account for the passed in server. Note that
 // destServer may not be "this"
-nsresult nsMsgIncomingServer::GetDeferredServers(nsIMsgIncomingServer *destServer, nsISupportsArray **_retval)
+nsresult nsMsgIncomingServer::GetDeferredServers(nsIMsgIncomingServer *destServer, nsCOMArray<nsIPop3IncomingServer>& aServers)
 {
   nsresult rv;
   nsCOMPtr<nsIMsgAccountManager> accountManager
     = do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
   NS_ENSURE_SUCCESS(rv, rv);
-  nsCOMPtr<nsISupportsArray> servers;
-  rv = NS_NewISupportsArray(getter_AddRefs(servers));
-  NS_ENSURE_SUCCESS(rv, rv);
 
   nsCOMPtr <nsIMsgAccount> thisAccount;
   accountManager->FindAccountForServer(destServer, getter_AddRefs(thisAccount));
   if (thisAccount)
   {
-    nsCOMPtr <nsISupportsArray> allServers;
+    nsCOMPtr<nsIArray> allServers;
     nsCString accountKey;
     thisAccount->GetKey(accountKey);
     accountManager->GetAllServers(getter_AddRefs(allServers));
     if (allServers)
     {
       uint32_t serverCount;
-      allServers->Count(&serverCount);
+      allServers->GetLength(&serverCount);
       for (uint32_t i = 0; i < serverCount; i++)
       {
-        nsCOMPtr <nsIMsgIncomingServer> server (do_QueryElementAt(allServers, i));
+        nsCOMPtr<nsIPop3IncomingServer> server(do_QueryElementAt(allServers, i));
         if (server)
         {
           nsCString deferredToAccount;
-          server->GetCharValue("deferred_to_account", deferredToAccount);
+          server->GetDeferredToAccount(deferredToAccount);
           if (deferredToAccount.Equals(accountKey))
-            servers->AppendElement(server);
+            aServers.AppendElement(server);
         }
       }
     }
   }
-  servers.swap(*_retval);
   return rv;
 }
 
@@ -2170,14 +2168,14 @@ NS_IMETHODIMP nsMsgIncomingServer::GetIsDeferredTo(bool *aIsDeferredTo)
     accountManager->FindAccountForServer(this, getter_AddRefs(thisAccount));
     if (thisAccount)
     {
-      nsCOMPtr <nsISupportsArray> allServers;
+      nsCOMPtr<nsIArray> allServers;
       nsCString accountKey;
       thisAccount->GetKey(accountKey);
       accountManager->GetAllServers(getter_AddRefs(allServers));
       if (allServers)
       {
         uint32_t serverCount;
-        allServers->Count(&serverCount);
+        allServers->GetLength(&serverCount);
         for (uint32_t i = 0; i < serverCount; i++)
         {
           nsCOMPtr <nsIMsgIncomingServer> server (do_QueryElementAt(allServers, i));
