@@ -33,8 +33,12 @@
 #include "nsMsgUtils.h"
 #include "nsILineInputStream.h"
 #include "nsIMsgIncomingServer.h"
+#include "nsMimeTypes.h"
 #include "nsAlgorithm.h"
 #include "mozilla/Services.h"
+#include <algorithm>
+
+#undef PostMessage // avoid to collision with WinUser.h
 
 using namespace mozilla;
 
@@ -632,8 +636,11 @@ NS_IMETHODIMP nsMsgProtocol::GetContentType(nsACString &aContentType)
 
 NS_IMETHODIMP nsMsgProtocol::SetContentType(const nsACString &aContentType)
 {
-    nsAutoCString charset;
-    return NS_ParseContentType(aContentType, m_ContentType, charset);
+  nsAutoCString charset;
+  nsresult rv = NS_ParseContentType(aContentType, m_ContentType, charset);
+  if (NS_FAILED(rv) || m_ContentType.IsEmpty())
+    m_ContentType.AssignLiteral(UNKNOWN_CONTENT_TYPE);
+  return rv;
 }
 
 NS_IMETHODIMP nsMsgProtocol::GetContentCharset(nsACString &aContentCharset)
@@ -678,13 +685,13 @@ nsMsgProtocol::GetContentDispositionHeader(nsACString &aContentDispositionHeader
   return NS_ERROR_NOT_AVAILABLE;
 }
 
-NS_IMETHODIMP nsMsgProtocol::GetContentLength(int32_t *aContentLength)
+NS_IMETHODIMP nsMsgProtocol::GetContentLength(int64_t *aContentLength)
 {
   *aContentLength = mContentLength;
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgProtocol::SetContentLength(int32_t aContentLength)
+NS_IMETHODIMP nsMsgProtocol::SetContentLength(int64_t aContentLength)
 {
   mContentLength = aContentLength;
   return NS_OK;
@@ -747,8 +754,8 @@ nsMsgProtocol::OnTransportStatus(nsITransport *transport, nsresult status,
     return NS_OK;
 
   // these transport events should not generate any status messages
-  if (status == nsISocketTransport::STATUS_RECEIVING_FROM ||
-      status == nsISocketTransport::STATUS_SENDING_TO)
+  if (status == NS_NET_STATUS_RECEIVING_FROM ||
+      status == NS_NET_STATUS_SENDING_TO)
     return NS_OK;
 
   if (!mProgressEventSink)
@@ -1080,7 +1087,7 @@ public:
 
         if (avail)
         {
-          rv = aOutStream->WriteFrom(mInStream, NS_MIN(avail, 4096), &bytesWritten);
+          rv = aOutStream->WriteFrom(mInStream, std::min(avail, uint64_t(4096)), &bytesWritten);
           // if were full at the time, the input stream may be backed up and we need to read any remains from the last ODA call
           // before we'll get more ODA calls
           if (protInst->mSuspendedRead)
@@ -1405,7 +1412,7 @@ nsresult nsMsgAsyncWriteProtocol::UnblockPostReader()
       uint64_t avail = 0;
       mPostDataStream->Available(&avail);
 
-      m_outputStream->WriteFrom(mPostDataStream, NS_MIN(avail, mSuspendedReadBytes), &amountWritten);
+      m_outputStream->WriteFrom(mPostDataStream, std::min(avail, uint64_t(mSuspendedReadBytes)), &amountWritten);
       // hmm sometimes my mSuspendedReadBytes is getting out of whack...so for now, reset it
       // if necessary.
       if (mSuspendedReadBytes > avail)
