@@ -34,7 +34,6 @@
 #include "nsIRelativeFilePref.h"
 #include "nsAppDirectoryServiceDefs.h"
 #include "nsISpamSettings.h"
-#include "nsISignatureVerifier.h"
 #include "nsICryptoHash.h"
 #include "nsNativeCharsetUtils.h"
 #include "nsDirectoryServiceUtils.h"
@@ -71,6 +70,7 @@
 #include "nsIDocumentEncoder.h"
 #include "mozilla/Services.h"
 #include "mozilla/Util.h"
+#include "locale.h"
 using namespace mozilla;
 
 static NS_DEFINE_CID(kImapUrlCID, NS_IMAPURL_CID);
@@ -288,7 +288,7 @@ static uint32_t StringHash(const char *ubuf, int32_t len = -1)
   uint32_t h=1;
   unsigned char *end = buf + (len == -1 ? strlen(ubuf) : len);
   while(buf < end) {
-    h = 0x63c63cd9*h + 0x9c39c33d + (int32)*buf;
+    h = 0x63c63cd9*h + 0x9c39c33d + (int32_t)*buf;
     buf++;
   }
   return h;
@@ -361,7 +361,7 @@ static bool ConvertibleToNative(const nsAutoString& str)
     return str.Equals(roundTripped);
 }
 
-#if defined(XP_UNIX) || defined(XP_BEOS)
+#if defined(XP_UNIX)
   const static uint32_t MAX_LEN = 55;
 #elif defined(XP_WIN32)
   const static uint32_t MAX_LEN = 55;
@@ -522,6 +522,26 @@ nsresult FormatFileSize(uint64_t size, bool useKB, nsAString &formattedSize)
   nsTextFormatter::ssprintf(
     formattedSize, sizeAbbr.get(),
     (unitIndex != 0) && (unitSize < 99.95 && unitSize != 0) ? 1 : 0, unitSize);
+
+  PRInt32 separatorPos = formattedSize.FindChar('.');
+  if (separatorPos != kNotFound) {
+    // The ssprintf returned a decimal number using a dot (.) as the decimal
+    // separator. Now we try to localize the separator.
+    // Try to get the decimal separator from the system's locale.
+    char *decimalPoint;
+#ifdef HAVE_LOCALECONV
+    struct lconv *locale = localeconv();
+    decimalPoint = locale->decimal_point;
+#else
+    decimalPoint = getenv("LOCALE_DECIMAL_POINT");
+#endif
+    NS_ConvertUTF8toUTF16 decimalSeparator(decimalPoint);
+    if (decimalSeparator.IsEmpty())
+      decimalSeparator.AssignLiteral(".");
+
+    formattedSize.Replace(separatorPos, 1, decimalSeparator);
+  }
+
   return NS_OK;
 }
 
@@ -629,8 +649,8 @@ bool NS_MsgStripRE(const char **stringP, uint32_t *lengthP, char **modifiedSubje
   {
     mimeConverter = do_GetService(NS_MIME_CONVERTER_CONTRACTID, &rv);
     if (NS_SUCCEEDED(rv))
-      rv = mimeConverter->DecodeMimeHeaderToCharPtr(
-        *stringP, nullptr, false, true, getter_Copies(decodedString));
+      rv = mimeConverter->DecodeMimeHeaderToUTF8(nsDependentCString(*stringP),
+        nullptr, false, true, decodedString);
   }
 
   s = !decodedString.IsEmpty() ? decodedString.get() : *stringP;
@@ -2192,9 +2212,10 @@ NS_MSG_BASE uint64_t ParseUint64Str(const char *str)
 NS_MSG_BASE nsresult
 MsgStreamMsgHeaders(nsIInputStream *aInputStream, nsIStreamListener *aConsumer)
 {
-  nsLineBuffer<char> *lineBuffer;
-  nsresult rv = NS_InitLineBuffer(&lineBuffer);
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsAutoPtr<nsLineBuffer<char> > lineBuffer(new nsLineBuffer<char>);
+  NS_ENSURE_TRUE(lineBuffer, NS_ERROR_OUT_OF_MEMORY);
+
+  nsresult rv;
 
   nsAutoCString msgHeaders;
   nsAutoCString curLine;
@@ -2204,14 +2225,14 @@ MsgStreamMsgHeaders(nsIInputStream *aInputStream, nsIStreamListener *aConsumer)
   // We want to NS_ReadLine until we get to a blank line (the end of the headers)
   while (more)
   {
-    rv = NS_ReadLine(aInputStream, lineBuffer, curLine, &more);
+    rv = NS_ReadLine(aInputStream, lineBuffer.get(), curLine, &more);
     NS_ENSURE_SUCCESS(rv, rv);
     if (curLine.IsEmpty())
       break;
     msgHeaders.Append(curLine);
     msgHeaders.Append(NS_LITERAL_CSTRING("\r\n"));
   }
-  PR_Free(lineBuffer);
+  lineBuffer = nullptr;
   nsCOMPtr<nsIStringInputStream> hdrsStream =
         do_CreateInstance("@mozilla.org/io/string-input-stream;1", &rv);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -2221,7 +2242,6 @@ MsgStreamMsgHeaders(nsIInputStream *aInputStream, nsIStreamListener *aConsumer)
   NS_ENSURE_SUCCESS(rv, rv);
 
   return pump->AsyncRead(aConsumer, nullptr);
-
 }
 
 class CharsetDetectionObserver : public nsICharsetDetectionObserver
