@@ -182,17 +182,6 @@ NS_IMETHODIMP nsMsgCompFields::GetNewsgroups(nsAString &aGroup)
   return GetUnicodeHeader(MSG_NEWSGROUPS_HEADER_ID, aGroup);
 }
 
-NS_IMETHODIMP nsMsgCompFields::SetNewshost(const char *value)
-{
-  return SetAsciiHeader(MSG_NEWSPOSTURL_HEADER_ID, value);
-}
-
-NS_IMETHODIMP nsMsgCompFields::GetNewshost(char **_retval)
-{
-  *_retval = strdup(GetAsciiHeader(MSG_NEWSPOSTURL_HEADER_ID));
-  return *_retval ? NS_OK : NS_ERROR_OUT_OF_MEMORY;
-}
-
 NS_IMETHODIMP nsMsgCompFields::SetFollowupTo(const nsAString &aValue)
 {
   return SetUnicodeHeader(MSG_FOLLOWUP_TO_HEADER_ID, aValue);
@@ -256,36 +245,6 @@ NS_IMETHODIMP nsMsgCompFields::GetOtherRandomHeaders(nsAString &_retval)
   return GetUnicodeHeader(MSG_OTHERRANDOMHEADERS_HEADER_ID, _retval);
 }
 
-NS_IMETHODIMP nsMsgCompFields::SetSenderReply(const nsAString &value)
-{
-  return SetUnicodeHeader(MSG_SENDERREPLY_HEADER_ID, value);
-}
-
-NS_IMETHODIMP nsMsgCompFields::GetSenderReply(nsAString &_retval)
-{
-  return GetUnicodeHeader(MSG_SENDERREPLY_HEADER_ID, _retval);
-}
-
-NS_IMETHODIMP nsMsgCompFields::SetAllReply(const nsAString &value)
-{
-  return SetUnicodeHeader(MSG_ALLREPLY_HEADER_ID, value);
-}
-
-NS_IMETHODIMP nsMsgCompFields::GetAllReply(nsAString &_retval)
-{
-  return GetUnicodeHeader(MSG_ALLREPLY_HEADER_ID, _retval);
-}
-
-NS_IMETHODIMP nsMsgCompFields::SetListReply(const nsAString &value)
-{
-  return SetUnicodeHeader(MSG_LISTREPLY_HEADER_ID, value);
-}
-
-NS_IMETHODIMP nsMsgCompFields::GetListReply(nsAString &_retval)
-{
-  return GetUnicodeHeader(MSG_LISTREPLY_HEADER_ID, _retval);
-}
-
 NS_IMETHODIMP nsMsgCompFields::SetNewspostUrl(const char *value)
 {
   return SetAsciiHeader(MSG_NEWSPOSTURL_HEADER_ID, value);
@@ -575,9 +534,9 @@ nsMsgCompFields::SplitRecipients(const nsAString &aRecipients,
       if (!aEmailAddressOnly)
       {
         nsCString decodedName;
-        converter->DecodeMimeHeaderToCharPtr(pNames, GetCharacterSet(),
-                                             false, true,
-                                             getter_Copies(decodedName));
+        converter->DecodeMimeHeaderToUTF8(nsDependentCString(pNames),
+                                          GetCharacterSet(), false, true,
+                                          decodedName);
         rv = parser->MakeFullAddressString((!decodedName.IsEmpty() ?
                                             decodedName.get() : pNames),
                                            pAddresses,
@@ -648,8 +607,9 @@ nsresult nsMsgCompFields::SplitRecipientsEx(const nsAString &recipients,
     {
       nsCString fullAddress;
       nsCString decodedName;
-      converter->DecodeMimeHeaderToCharPtr(pNames, GetCharacterSet(), false, true,
-                                           getter_Copies(decodedName));
+      converter->DecodeMimeHeaderToUTF8(nsDependentCString(pNames),
+                                        GetCharacterSet(), false, true,
+                                        decodedName);
       rv = parser->MakeFullAddressString((!decodedName.IsEmpty() ?
                                           decodedName.get() : pNames),
                                          pAddresses,
