@@ -500,7 +500,7 @@ nsMsgMailboxParser::ReleaseFolderLock()
   bool haveSemaphore;
   nsCOMPtr <nsISupports> supports = do_QueryInterface(static_cast<nsIMsgParseMailMsgState*>(this));
   result = folder->TestSemaphore(supports, &haveSemaphore);
-  if(NS_SUCCEEDED(result) && haveSemaphore)
+  if (NS_SUCCEEDED(result) && haveSemaphore)
     (void) folder->ReleaseSemaphore(supports);
 }
 
@@ -935,10 +935,11 @@ nsresult nsParseMailMessageState::StartNewEnvelope(const char *line, uint32_t li
 nsresult nsParseMailMessageState::ParseHeaders ()
 {
   char *buf = m_headers.GetBuffer();
-  char *buf_end = buf + m_headers.GetBufferPos();
+  uint32_t buf_length = m_headers.GetBufferPos();
+  char *buf_end = buf + buf_length;
   while (buf < buf_end)
   {
-    char *colon = PL_strchr (buf, ':');
+    char *colon = PL_strnchr(buf, ':', buf_length);
     char *end;
     char *value = 0;
     struct message_header *header = 0;
@@ -1051,8 +1052,8 @@ nsresult nsParseMailMessageState::ParseHeaders ()
 #endif
 
       ToLowerCase(headerStr);
-      int32_t customHeaderIndex = m_customDBHeaders.IndexOf(headerStr);
-      if (customHeaderIndex != -1)
+      uint32_t customHeaderIndex = m_customDBHeaders.IndexOf(headerStr);
+      if (customHeaderIndex != nsTArray<nsCString>::NoIndex)
         header = & m_customDBHeaderValues[customHeaderIndex];
     }
 
@@ -1144,13 +1145,15 @@ SEARCH_NEWLINE:
             PRTime resultTime;
             if (PR_ParseTimeString (receivedDate.get(), false, &resultTime) == PR_SUCCESS)
               m_receivedTime = resultTime;
+            else
+              NS_WARNING("PR_ParseTimeString failed in ParseHeaders().");
           }
         }
         // Someone might want the received header saved.
         if (m_customDBHeaders.Length())
         {
           uint32_t customHeaderIndex = m_customDBHeaders.IndexOf(NS_LITERAL_CSTRING("received"));
-          if (customHeaderIndex != -1)
+          if (customHeaderIndex != nsTArray<nsCString>::NoIndex)
           {
             if (!m_receivedValue.IsEmpty())
               m_receivedValue.Append(' ');
@@ -1183,7 +1186,9 @@ nsresult nsParseMailMessageState::ParseEnvelope (const char *line, uint32_t line
     s++;
   m_envelope_date.value = s;
   m_envelope_date.length = (uint16_t) (line_size - (s - m_envelope.GetBuffer()));
-  while (IS_SPACE (m_envelope_date.value [m_envelope_date.length - 1]))
+
+  while (m_envelope_date.length > 0 &&
+         IS_SPACE (m_envelope_date.value [m_envelope_date.length - 1]))
     m_envelope_date.length--;
 
   /* #### short-circuit const */
@@ -1362,9 +1367,12 @@ nsresult nsParseMailMessageState::FinalizeHeaders()
     /* Take off <> around message ID. */
     if (id)
     {
-      if (id->value[0] == '<')
-        id->value++, id->length--;
-      if (id->value[id->length - 1] == '>')
+      if (id->length > 0 && id->value[0] == '<')
+        id->length--, id->value++;
+
+      NS_WARN_IF_FALSE(id->length > 0, "id->length failure in FinalizeHeaders().");
+
+      if (id->length > 0 && id->value[id->length - 1] == '>')
         /* generate a new null-terminated string without the final > */
         rawMsgId.Assign(id->value, id->length - 1);
       else
@@ -1463,7 +1471,7 @@ nsresult nsParseMailMessageState::FinalizeHeaders()
         ret = m_HeaderAddressParser->ParseHeaderAddresses(recipient->value,
                                                           &names, &addresses,
                                                           &numAddresses);
-        if (ret == NS_OK)
+        if (NS_SUCCEEDED(ret))
         {
           m_newMsgHdr->SetRecipientsArray(names, addresses, numAddresses);
           PR_Free(addresses);
@@ -1558,6 +1566,9 @@ nsresult nsParseMailMessageState::FinalizeHeaders()
             case 'U': case 'u':
               m_newMsgHdr->SetFlags(msgFlags & ~nsMsgMessageFlags::Read);
               break;
+            default:            // Should check for corrupt file.
+              NS_ERROR("Corrupt file. Should not happen.");
+              break;
             }
           }
         }
@@ -1591,6 +1602,8 @@ nsresult nsParseMailMessageState::FinalizeHeaders()
             m_newMsgHdr->SetDate(resultTime);
             PRTime2Seconds(resultTime, &rcvTimeSecs);
           }
+          else
+            NS_WARNING("PR_ParseTimeString of date failed in FinalizeHeader().");
         }
         else
         {  // PR_Now()
@@ -1613,6 +1626,8 @@ nsresult nsParseMailMessageState::FinalizeHeaders()
           PRStatus timeStatus = PR_ParseTimeString (deliveryDate->value, false, &resultTime);
           if (PR_SUCCESS == timeStatus)
             PRTime2Seconds(resultTime, &rcvTimeSecs);
+          else // TODO/FIXME: We need to figure out what to do in this case!
+            NS_WARNING("PR_ParseTimeString of delivery date failed in FinalizeHeader().");
         }
         m_newMsgHdr->SetUint32Property("dateReceived", rcvTimeSecs);
 
@@ -1931,7 +1946,7 @@ nsresult nsParseNewMailState::GetTrashFolder(nsIMsgFolder **pTrashFolder)
   if (!pTrashFolder)
     return NS_ERROR_NULL_POINTER;
 
-  if(m_downloadFolder)
+  if (m_downloadFolder)
   {
     nsCOMPtr <nsIMsgIncomingServer> incomingServer;
     m_downloadFolder->GetServer(getter_AddRefs(incomingServer));
@@ -1986,10 +2001,9 @@ void nsParseNewMailState::ApplyFilters(bool *pMoved, nsIMsgWindow *msgWindow, ui
 
 NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWindow *msgWindow, bool *applyMore)
 {
+  NS_ENSURE_ARG_POINTER(filter);
   NS_ENSURE_ARG_POINTER(applyMore);
 
-  nsMsgRuleActionType actionType;
-  nsCString actionTargetFolderUri;
   uint32_t newFlags;
   nsresult rv = NS_OK;
 
@@ -1997,14 +2011,13 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
 
   nsCOMPtr<nsIMsgDBHdr> msgHdr = m_newMsgHdr;
 
-  nsCOMPtr<nsISupportsArray> filterActionList;
-  rv = NS_NewISupportsArray(getter_AddRefs(filterActionList));
-  NS_ENSURE_SUCCESS(rv, rv);
-  rv = filter->GetSortedActionList(filterActionList);
+  nsCOMPtr<nsIArray> filterActionList;
+
+  rv = filter->GetSortedActionList(getter_AddRefs(filterActionList));
   NS_ENSURE_SUCCESS(rv, rv);
 
   uint32_t numActions;
-  rv = filterActionList->Count(&numActions);
+  rv = filterActionList->GetLength(&numActions);
   NS_ENSURE_SUCCESS(rv, rv);
 
   bool loggingEnabled = false;
@@ -2012,20 +2025,24 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
     m_filterList->GetLoggingEnabled(&loggingEnabled);
 
   bool msgIsNew = true;
-  for (uint32_t actionIndex =0; actionIndex < numActions && *applyMore; actionIndex++)
+  for (uint32_t actionIndex = 0; actionIndex < numActions && *applyMore; actionIndex++)
   {
     nsCOMPtr<nsIMsgRuleAction> filterAction;
-    filterActionList->QueryElementAt(actionIndex, NS_GET_IID(nsIMsgRuleAction), getter_AddRefs(filterAction));
-    if (!filterAction)
+    rv = filterActionList->QueryElementAt(actionIndex, NS_GET_IID(nsIMsgRuleAction),
+                                                       getter_AddRefs(filterAction));
+    if (NS_FAILED(rv) || !filterAction)
       continue;
 
+    nsMsgRuleActionType actionType;
     if (NS_SUCCEEDED(filterAction->GetType(&actionType)))
     {
+      nsCString actionTargetFolderUri;
       if (actionType == nsMsgFilterAction::MoveToFolder ||
           actionType == nsMsgFilterAction::CopyToFolder)
       {
-        filterAction->GetTargetFolderUri(actionTargetFolderUri);
-        if (actionTargetFolderUri.IsEmpty())
+
+        rv = filterAction->GetTargetFolderUri(actionTargetFolderUri);
+        if (NS_FAILED(rv) || actionTargetFolderUri.IsEmpty())
         {
           NS_ASSERTION(false, "actionTargetFolderUri is empty");
           continue;
@@ -2578,7 +2595,7 @@ nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr,
 
   if (movedMsgIsNew)
     destIFolder->SetHasNewMessages(true);
-  if (m_filterTargetFolders.IndexOf(destIFolder) == -1)
+  if (m_filterTargetFolders.IndexOf(destIFolder) == -1) // IndexOf() type?
     m_filterTargetFolders.AppendObject(destIFolder);
 
   destIFolder->ReleaseSemaphore (myISupports);
