@@ -52,7 +52,6 @@
 #include "prsystem.h"
 #include "nsIArray.h"
 #include "nsIServiceManager.h"
-#include "nsIEnumerator.h"
 #include "nsINntpService.h"
 #include "nsIFolderListener.h"
 #include "nsCOMPtr.h"
@@ -167,9 +166,9 @@ void AsyncAuthMigrator::EnqueuePrompt()
 
 nsMsgNewsFolder::nsMsgNewsFolder(void) :
      mExpungedBytes(0), mGettingNews(false),
-    mInitialized(false),
-    m_downloadMessageForOfflineUse(false), m_downloadingMultipleMessages(false),
-    mReadSet(nullptr)
+     mInitialized(false),
+     m_downloadMessageForOfflineUse(false), m_downloadingMultipleMessages(false),
+     mReadSet(nullptr), mSortOrder(kNewsSortOffset)
 {
   MOZ_COUNT_CTOR(nsNewsFolder); // double count these for now.
 }
@@ -844,11 +843,6 @@ NS_IMETHODIMP nsMsgNewsFolder::GetDeletable(bool *deletable)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsMsgNewsFolder::GetRequiresCleanup(bool *requiresCleanup)
-{
-  return NS_ERROR_NOT_IMPLEMENTED;
-}
-
 NS_IMETHODIMP nsMsgNewsFolder::GetSizeOnDisk(uint32_t *size)
 {
   return NS_ERROR_NOT_IMPLEMENTED;
