@@ -45,13 +45,13 @@
 
 // this platform specific junk is so the newsrc filenames we create
 // will resemble the migrated newsrc filenames.
-#if defined(XP_UNIX) || defined(XP_BEOS)
+#if defined(XP_UNIX)
 #define NEWSRC_FILE_PREFIX "newsrc-"
 #define NEWSRC_FILE_SUFFIX ""
 #else
 #define NEWSRC_FILE_PREFIX ""
 #define NEWSRC_FILE_SUFFIX ".rc"
-#endif /* XP_UNIX || XP_BEOS */
+#endif /* XP_UNIX */
 
 // ###tw  This really ought to be the most
 // efficient file reading size for the current
@@ -93,6 +93,8 @@ nsNntpIncomingServer::nsNntpIncomingServer()
 {
   mNewsrcHasChanged = false;
 
+  mGetOnlyNew = true;
+
   mHostInfoLoaded = false;
   mHostInfoHasChanged = false;
   mVersion = INVALID_VERSION;
@@ -723,10 +725,12 @@ nsNntpIncomingServer::OnStopRunningUrl(nsIURI *url, nsresult exitCode)
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::ContainsNewsgroup(const nsACString &name,
+nsNntpIncomingServer::ContainsNewsgroup(const nsACString &aName,
                                         bool *containsGroup)
 {
-    if (name.IsEmpty()) return NS_ERROR_FAILURE;
+    NS_ENSURE_ARG_POINTER(containsGroup);
+    NS_ENSURE_FALSE(aName.IsEmpty(), NS_ERROR_FAILURE);
+
     if (mSubscribedNewsgroups.Length() == 0)
     {
       // If this is empty, we may need to discover folders
@@ -739,8 +743,8 @@ nsNntpIncomingServer::ContainsNewsgroup(const nsACString &name,
       }
     }
     nsAutoCString unescapedName;
-    MsgUnescapeString(name, 0, unescapedName);
-    *containsGroup = mSubscribedNewsgroups.Contains(name);
+    MsgUnescapeString(aName, 0, unescapedName);
+    *containsGroup = mSubscribedNewsgroups.Contains(aName);
     return NS_OK;
 }
 
@@ -748,17 +752,21 @@ NS_IMETHODIMP
 nsNntpIncomingServer::SubscribeToNewsgroup(const nsACString &aName)
 {
     NS_ASSERTION(!aName.IsEmpty(), "no name");
-    if (aName.IsEmpty()) return NS_ERROR_FAILURE;
+    NS_ENSURE_FALSE(aName.IsEmpty(), NS_ERROR_FAILURE);
+
+    // If we already have this newsgroup, do nothing and report success.
+    bool containsGroup = false;
+    nsresult rv = ContainsNewsgroup(aName, &containsGroup);
+    NS_ENSURE_SUCCESS(rv, rv);
+    if (containsGroup)
+      return NS_OK;
 
     nsCOMPtr<nsIMsgFolder> msgfolder;
-    nsresult rv = GetRootMsgFolder(getter_AddRefs(msgfolder));
-    if (NS_FAILED(rv)) return rv;
-    if (!msgfolder) return NS_ERROR_FAILURE;
+    rv = GetRootMsgFolder(getter_AddRefs(msgfolder));
+    NS_ENSURE_SUCCESS(rv, rv);
+    NS_ENSURE_TRUE(msgfolder, NS_ERROR_FAILURE);
 
-    rv = msgfolder->CreateSubfolder(NS_ConvertUTF8toUTF16(aName), nullptr);
-    if (NS_FAILED(rv)) return rv;
-
-    return NS_OK;
+    return msgfolder->CreateSubfolder(NS_ConvertUTF8toUTF16(aName), nullptr);
 }
 
 bool
@@ -1198,14 +1206,16 @@ nsNntpIncomingServer::Unsubscribe(const PRUnichar *aUnicharName)
   return NS_OK;
 }
 
-int32_t
+nsresult
 nsNntpIncomingServer::HandleLine(const char* line, uint32_t line_size)
 {
   NS_ASSERTION(line, "line is null");
-  if (!line) return 0;
+  if (!line)
+    return NS_OK;
 
   // skip blank lines and comments
-  if (line[0] == '#' || line[0] == '\0') return 0;
+  if (line[0] == '#' || line[0] == '\0')
+    return NS_OK;
   // ###TODO - make this truly const, maybe pass in an nsCString &
 
   if (mHasSeenBeginGroups) {
@@ -1245,7 +1255,7 @@ nsNntpIncomingServer::HandleLine(const char* line, uint32_t line_size)
     }
   }
 
-  return 0;
+  return NS_OK;
 }
 
 nsresult
@@ -1699,19 +1709,18 @@ nsNntpIncomingServer::SetSelection(nsITreeSelection * aSelection)
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::GetRowProperties(int32_t index, nsISupportsArray *properties)
+nsNntpIncomingServer::GetRowProperties(int32_t index, nsAString& properties)
 {
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::GetCellProperties(int32_t row, nsITreeColumn* col, nsISupportsArray *properties)
+nsNntpIncomingServer::GetCellProperties(int32_t row, nsITreeColumn* col, nsAString& properties)
 {
     if (!IsValidRow(row))
       return NS_ERROR_UNEXPECTED;
 
     NS_ENSURE_ARG_POINTER(col);
-    NS_ENSURE_ARG_POINTER(properties);
 
     const PRUnichar* colID;
     col->GetIdConst(&colID);
@@ -1722,19 +1731,19 @@ nsNntpIncomingServer::GetCellProperties(int32_t row, nsITreeColumn* col, nsISupp
         if (mSearchResultSortDescending)
           row = mSubscribeSearchResult.Length() - 1 - row;
         if (mTempSubscribed.IndexOf(mSubscribeSearchResult.ElementAt(row)) != mTempSubscribed.NoIndex) {
-          properties->AppendElement(mSubscribedAtom);
+          properties.AssignLiteral("subscribed");
         }
     }
     else if (colID[0] == 'n') {
       // add the "nntp" property to the "nameCol"
       // so we get the news folder icon in the search view
-      properties->AppendElement(mNntpAtom);
+      properties.AssignLiteral("nntp");
     }
     return NS_OK;
 }
 
 NS_IMETHODIMP
-nsNntpIncomingServer::GetColumnProperties(nsITreeColumn* col, nsISupportsArray *properties)
+nsNntpIncomingServer::GetColumnProperties(nsITreeColumn* col, nsAString& properties)
 {
     return NS_OK;
 }
@@ -2128,3 +2137,11 @@ nsNntpIncomingServer::OnUserOrHostNameChanged(const nsACString& oldName,
   // Force updating the rc file.
   return CommitSubscribeChanges();
 }
+
+NS_IMETHODIMP
+nsNntpIncomingServer::GetSortOrder(int32_t* aSortOrder)
+{
+  NS_ENSURE_ARG_POINTER(aSortOrder);
+  *aSortOrder = 500000000;
+  return NS_OK;
+}
