@@ -1,40 +1,8 @@
-/* ***** BEGIN LICENSE BLOCK *****
- *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Thunderbird Mail Client.
- *
- * The Initial Developer of the Original Code is
- * the Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2009
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Andrew Sutherland <asutherland@asutherland.org>
- *   Mike Conley <mconley@mozilla.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+
+Components.utils.import("resource:///modules/mailServices.js");
 
 var gMessageGenerator, gMessageScenarioFactory;
 
@@ -60,23 +28,16 @@ function configure_message_injection(aInjectionConfig) {
   mis.injectionConfig = aInjectionConfig;
 
   // Disable new mail notifications
-  var prefSvc = Cc["@mozilla.org/preferences-service;1"]
-                  .getService(Ci.nsIPrefBranch);
-
-  prefSvc.setBoolPref("mail.biff.play_sound", false);
-  prefSvc.setBoolPref("mail.biff.show_alert", false);
-  prefSvc.setBoolPref("mail.biff.show_tray_icon", false);
-  prefSvc.setBoolPref("mail.biff.animate_dock_icon", false);
+  Services.prefs.setBoolPref("mail.biff.play_sound", false);
+  Services.prefs.setBoolPref("mail.biff.show_alert", false);
+  Services.prefs.setBoolPref("mail.biff.show_tray_icon", false);
+  Services.prefs.setBoolPref("mail.biff.animate_dock_icon", false);
 
 
   // we need to pull in the notification service so we get events?
-  mis.mfnService = Cc["@mozilla.org/messenger/msgnotificationservice;1"]
-                     .getService(Ci.nsIMsgFolderNotificationService);
+  mis.mfnService = MailServices.mfn;
 
 
-  let acctMgr = Cc["@mozilla.org/messenger/account-manager;1"]
-                  .getService(Ci.nsIMsgAccountManager);
-
   if (mis.injectionConfig.mode == "pop") {
     // -- Pull in the POP3 fake-server / local account helper code
     load(gDEPTH + "mailnews/local/test/unit/head_maillocal.js");
@@ -91,8 +52,7 @@ function configure_message_injection(aInjectionConfig) {
     mis.rootFolder = mis.incomingServer.rootMsgFolder;
     mis.inboxFolder = mis.rootFolder.getChildNamed("Inbox");
 
-    mis.pop3Service = Cc["@mozilla.org/messenger/popservice;1"]
-      .getService(Ci.nsIPop3Service);
+    mis.pop3Service = MailServices.pop3;
 
     mis.server.start(POP3_PORT);
   }
@@ -100,24 +60,24 @@ function configure_message_injection(aInjectionConfig) {
     // This does createIncomingServer() and createAccount(), sets the server as
     //  the account's server, then sets the server
     try {
-      acctMgr.createLocalMailAccount();
+      MailServices.accounts.createLocalMailAccount();
     }
     catch (ex) {
       // This will fail if someone already called this.  Like in the mozmill
       //  case.
     }
 
-    let localAccount = acctMgr.FindAccountForServer(acctMgr.localFoldersServer);
+    let localAccount = MailServices.accounts.FindAccountForServer(MailServices.accounts.localFoldersServer);
 
     // We need an identity or we get angry warnings.
-    let identity = acctMgr.createIdentity();
+    let identity = MailServices.accounts.createIdentity();
     // We need an email to protect against random code assuming it exists and
     // throwing exceptions.
-    identity.email = "sender@nul.nul";
+    identity.email = "sender@nul.invalid";
     localAccount.addIdentity(identity);
     localAccount.defaultIdentity = identity;
 
-    mis.incomingServer = acctMgr.localFoldersServer;
+    mis.incomingServer = MailServices.accounts.localFoldersServer;
     // Note: Inbox is not created automatically when there is no deferred server,
     // so we need to create it.
     mis.rootFolder = mis.incomingServer.rootMsgFolder;
@@ -133,18 +93,17 @@ function configure_message_injection(aInjectionConfig) {
     let unused = mis.inboxFolder.prettiestName;
   }
   else if (mis.injectionConfig.mode == "imap") {
-    const gPrefs = Cc["@mozilla.org/preferences-service;1"]
-                     .getService(Ci.nsIPrefBranch);
     // Disable autosync in favor of our explicitly forcing downloads of all
     //  messages in a folder.  This is being done speculatively because when we
     //  didn't do this we got tripped up by the semaphore being in use and
     //  concern over inability to hang a listener off of the completion of the
     //  download.  (Although I'm sure there are various ways we could do it.)
-    gPrefs.setBoolPref("mail.server.default.autosync_offline_stores", false);
+    Services.prefs.setBoolPref("mail.server.default.autosync_offline_stores",
+                               false);
     // Set the offline property based on the configured setting.  This will
     //  affect newly created folders.
-    gPrefs.setBoolPref("mail.server.default.offline_download",
-                       mis.injectionConfig.offline);
+    Services.prefs.setBoolPref("mail.server.default.offline_download",
+                               mis.injectionConfig.offline);
 
     // Pull in the IMAP fake server code
     load(gDEPTH + "mailnews/imap/test/unit/head_server.js");
@@ -159,29 +118,29 @@ function configure_message_injection(aInjectionConfig) {
     mis.server._logTransactions = false;
 
     // we need a local account for the IMAP server to have its sent messages in
-    acctMgr.createLocalMailAccount();
+    MailServices.accounts.createLocalMailAccount();
 
     // We need an identity so that updateFolder doesn't fail
-    let localAccount = acctMgr.createAccount();
-    let identity = acctMgr.createIdentity();
+    let localAccount = MailServices.accounts.createAccount();
+    let identity = MailServices.accounts.createIdentity();
     // We need an email to protect against random code assuming it exists and
     // throwing exceptions.
-    identity.email = "sender@nul.nul";
+    identity.email = "sender@nul.invalid";
     localAccount.addIdentity(identity);
     localAccount.defaultIdentity = identity;
     localAccount.incomingServer = mis.incomingServer;
-    acctMgr.defaultAccount = localAccount;
+    MailServices.accounts.defaultAccount = localAccount;
 
     // Let's also have another account, using the same identity
-    let imapAccount = acctMgr.createAccount();
+    let imapAccount = MailServices.accounts.createAccount();
     imapAccount.addIdentity(identity);
     imapAccount.defaultIdentity = identity;
     imapAccount.incomingServer = mis.incomingServer;
 
     // The server doesn't support more than one connection
-    prefSvc.setIntPref("mail.server.server1.max_cached_connections", 1);
+    Services.prefs.setIntPref("mail.server.server1.max_cached_connections", 1);
     // We aren't interested in downloading messages automatically
-    prefSvc.setBoolPref("mail.server.server1.download_on_biff", false);
+    Services.prefs.setBoolPref("mail.server.server1.download_on_biff", false);
 
     mis.rootFolder = mis.incomingServer.rootMsgFolder;
 
@@ -195,8 +154,7 @@ function configure_message_injection(aInjectionConfig) {
     _messageInjectionSetup.notifyListeners("onRealFolderCreated",
                                            [mis.inboxFolder]);
 
-    mis.imapService = Cc["@mozilla.org/messenger/imapservice;1"]
-                        .getService(Ci.nsIImapService);
+    mis.imapService = MailServices.imap;
 
     mis.handleUriToRealFolder = {};
     mis.handleUriToFakeFolder = {};
@@ -396,11 +354,9 @@ function make_empty_folder(aFolderName, aSpecialFlags) {
 
 // Small helper for moving folder. You have to yield move_folder(f1, f2);
 function move_folder(aSource, aTarget) {
-  let cs = Cc["@mozilla.org/messenger/messagecopyservice;1"]
-          .getService(Ci.nsIMsgCopyService);
   let array = toXPCOMArray([get_nsIMsgFolder(aSource)], Ci.nsIMutableArray);
   // we're doing a true move
-  cs.CopyFolders(array, get_nsIMsgFolder(aTarget), true, {
+  MailServices.copy.CopyFolders(array, get_nsIMsgFolder(aTarget), true, {
     /* nsIMsgCopyServiceListener implementation */
     OnStartCopy: function() {},
     OnProgress: function(aProgress, aProgressMax) {},
@@ -924,8 +880,6 @@ function async_move_messages(aSynMessageSet, aDestFolder, aAllowUndo) {
       // and then we can make sure we have the actual folder
       let realDestFolder = get_real_injection_folder(aDestFolder);
 
-      let copyService = Cc["@mozilla.org/messenger/messagecopyservice;1"]
-                          .getService(Ci.nsIMsgCopyService);
       for (let [folder, xpcomHdrArray] in
            aSynMessageSet.foldersWithXpcomHdrArrays) {
         mark_action("messageInjection",
@@ -938,10 +892,10 @@ function async_move_messages(aSynMessageSet, aDestFolder, aAllowUndo) {
           _messageInjectionSetup.notifyListeners(
             "onMovingMessagesWithoutDestHeaders", [realDestFolder]);
 
-        copyService.CopyMessages(folder, xpcomHdrArray,
-                                 realDestFolder, /* move */ true,
-                                 asyncCopyListener, null,
-                                 Boolean(aAllowUndo));
+        MailServices.copy.CopyMessages(folder, xpcomHdrArray,
+                                       realDestFolder, /* move */ true,
+                                       asyncCopyListener, null,
+                                       Boolean(aAllowUndo));
         // update the synthetic message set's folder entry...
         aSynMessageSet._folderSwap(folder, realDestFolder);
         yield false;
