@@ -14,7 +14,9 @@
 #include "jsprf.h"
 #include "jswrapper.h"
 
+#include "builtin/TestingFunctions.h"
 #include "methodjit/MethodJIT.h"
+#include "vm/ForkJoin.h"
 
 #include "vm/Stack-inl.h"
 
@@ -63,6 +65,30 @@ GetBuildConfiguration(JSContext *cx, unsigned argc, jsval *vp)
     if (!JS_SetProperty(cx, info, "has-ctypes", &value))
         return false;
 
+#ifdef JS_CPU_X86
+    value = BooleanValue(true);
+#else
+    value = BooleanValue(false);
+#endif
+    if (!JS_SetProperty(cx, info, "x86", &value))
+        return false;
+
+#ifdef JS_CPU_X64
+    value = BooleanValue(true);
+#else
+    value = BooleanValue(false);
+#endif
+    if (!JS_SetProperty(cx, info, "x64", &value))
+        return false;
+
+#ifdef MOZ_ASAN
+    value = BooleanValue(true);
+#else
+    value = BooleanValue(false);
+#endif
+    if (!JS_SetProperty(cx, info, "asan", &value))
+        return false;
+
 #ifdef JS_GC_ZEAL
     value = BooleanValue(true);
 #else
@@ -151,12 +177,12 @@ GetBuildConfiguration(JSContext *cx, unsigned argc, jsval *vp)
     if (!JS_SetProperty(cx, info, "methodjit", &value))
         return false;
 
-#ifdef JS_HAS_XML_SUPPORT
+#ifdef ENABLE_PARALLEL_JS
     value = BooleanValue(true);
 #else
     value = BooleanValue(false);
 #endif
-    if (!JS_SetProperty(cx, info, "e4x", &value))
+    if (!JS_SetProperty(cx, info, "parallelJS", &value))
         return false;
 
     *vp = ObjectValue(*info);
@@ -179,7 +205,7 @@ GC(JSContext *cx, unsigned argc, jsval *vp)
             if (!JS_StringEqualsAscii(cx, arg.toString(), "compartment", &compartment))
                 return false;
         } else if (arg.isObject()) {
-            PrepareCompartmentForGC(UnwrapObject(&arg.toObject())->compartment());
+            PrepareZoneForGC(UnwrapObject(&arg.toObject())->zone());
             compartment = true;
         }
     }
@@ -326,6 +352,23 @@ InternalConst(JSContext *cx, unsigned argc, jsval *vp)
     return true;
 }
 
+static JSBool
+GCPreserveCode(JSContext *cx, unsigned argc, jsval *vp)
+{
+    CallArgs args = CallArgsFromVp(argc, vp);
+
+    if (argc != 0) {
+        RootedObject callee(cx, &args.callee());
+        ReportUsageError(cx, callee, "Wrong number of arguments");
+        return JS_FALSE;
+    }
+
+    cx->runtime->alwaysPreserveCode = true;
+
+    *vp = JSVAL_VOID;
+    return JS_TRUE;
+}
+
 #ifdef JS_GC_ZEAL
 static JSBool
 GCZeal(JSContext *cx, unsigned argc, jsval *vp)
@@ -364,12 +407,12 @@ ScheduleGC(JSContext *cx, unsigned argc, jsval *vp)
         /* Schedule a GC to happen after |arg| allocations. */
         JS_ScheduleGC(cx, args[0].toInt32());
     } else if (args[0].isObject()) {
-        /* Ensure that |comp| is collected during the next GC. */
-        JSCompartment *comp = UnwrapObject(&args[0].toObject())->compartment();
-        PrepareCompartmentForGC(comp);
+        /* Ensure that |zone| is collected during the next GC. */
+        Zone *zone = UnwrapObject(&args[0].toObject())->zone();
+        PrepareZoneForGC(zone);
     } else if (args[0].isString()) {
         /* This allows us to schedule atomsCompartment for GC. */
-        PrepareCompartmentForGC(args[0].toString()->compartment());
+        PrepareZoneForGC(args[0].toString()->zone());
     }
 
     *vp = JSVAL_VOID;
@@ -421,6 +464,52 @@ VerifyPostBarriers(JSContext *cx, unsigned argc, jsval *vp)
     return JS_TRUE;
 }
 
+static JSBool
+GCState(JSContext *cx, unsigned argc, jsval *vp)
+{
+    CallArgs args = CallArgsFromVp(argc, vp);
+
+    if (argc != 0) {
+        RootedObject callee(cx, &args.callee());
+        ReportUsageError(cx, callee, "Too many arguments");
+        return false;
+    }
+
+    const char *state;
+    gc::State globalState = cx->runtime->gcIncrementalState;
+    if (globalState == gc::NO_INCREMENTAL)
+        state = "none";
+    else if (globalState == gc::MARK)
+        state = "mark";
+    else if (globalState == gc::SWEEP)
+        state = "sweep";
+    else
+        JS_NOT_REACHED("Unobserveable global GC state");
+
+    JSString *str = JS_NewStringCopyZ(cx, state);
+    if (!str)
+        return false;
+    *vp = StringValue(str);
+    return true;
+}
+
+static JSBool
+DeterministicGC(JSContext *cx, unsigned argc, jsval *vp)
+{
+    CallArgs args = CallArgsFromVp(argc, vp);
+
+    if (argc != 1) {
+        RootedObject callee(cx, &args.callee());
+        ReportUsageError(cx, callee, "Wrong number of arguments");
+        return JS_FALSE;
+    }
+
+    gc::SetDeterministicGC(cx, ToBoolean(vp[2]));
+    *vp = JSVAL_VOID;
+    return JS_TRUE;
+}
+#endif /* JS_GC_ZEAL */
+
 static JSBool
 GCSlice(JSContext *cx, unsigned argc, jsval *vp)
 {
@@ -446,40 +535,6 @@ GCSlice(JSContext *cx, unsigned argc, jsval *vp)
     return JS_TRUE;
 }
 
-static JSBool
-GCPreserveCode(JSContext *cx, unsigned argc, jsval *vp)
-{
-    CallArgs args = CallArgsFromVp(argc, vp);
-
-    if (argc != 0) {
-        RootedObject callee(cx, &args.callee());
-        ReportUsageError(cx, callee, "Wrong number of arguments");
-        return JS_FALSE;
-    }
-
-    cx->runtime->alwaysPreserveCode = true;
-
-    *vp = JSVAL_VOID;
-    return JS_TRUE;
-}
-
-static JSBool
-DeterministicGC(JSContext *cx, unsigned argc, jsval *vp)
-{
-    CallArgs args = CallArgsFromVp(argc, vp);
-
-    if (argc != 1) {
-        RootedObject callee(cx, &args.callee());
-        ReportUsageError(cx, callee, "Wrong number of arguments");
-        return JS_FALSE;
-    }
-
-    gc::SetDeterministicGC(cx, ToBoolean(vp[2]));
-    *vp = JSVAL_VOID;
-    return JS_TRUE;
-}
-#endif /* JS_GC_ZEAL */
-
 static JSBool
 ValidateGC(JSContext *cx, unsigned argc, jsval *vp)
 {
@@ -496,6 +551,51 @@ ValidateGC(JSContext *cx, unsigned argc, jsval *vp)
     return JS_TRUE;
 }
 
+static JSBool
+FullCompartmentChecks(JSContext *cx, unsigned argc, jsval *vp)
+{
+    CallArgs args = CallArgsFromVp(argc, vp);
+
+    if (argc != 1) {
+        RootedObject callee(cx, &args.callee());
+        ReportUsageError(cx, callee, "Wrong number of arguments");
+        return JS_FALSE;
+    }
+
+    gc::SetFullCompartmentChecks(cx, ToBoolean(vp[2]));
+    *vp = JSVAL_VOID;
+    return JS_TRUE;
+}
+
+static JSBool
+NondeterminsticGetWeakMapKeys(JSContext *cx, unsigned argc, jsval *vp)
+{
+    CallArgs args = CallArgsFromVp(argc, vp);
+
+    if (argc != 1) {
+        RootedObject callee(cx, &args.callee());
+        ReportUsageError(cx, callee, "Wrong number of arguments");
+        return false;
+    }
+    if (!args[0].isObject()) {
+        JS_ReportErrorNumber(cx, js_GetErrorMessage, NULL, JSMSG_NOT_EXPECTED_TYPE,
+                             "nondeterministicGetWeakMapKeys", "WeakMap",
+                             InformalValueTypeName(args[0]));
+        return false;
+    }
+    RootedObject arr(cx);
+    if (!JS_NondeterministicGetWeakMapKeys(cx, &args[0].toObject(), arr.address()))
+        return false;
+    if (!arr) {
+        JS_ReportErrorNumber(cx, js_GetErrorMessage, NULL, JSMSG_NOT_EXPECTED_TYPE,
+                             "nondeterministicGetWeakMapKeys", "WeakMap",
+                             args[0].toObject().getClass()->name);
+        return false;
+    }
+    args.rval().setObject(*arr);
+    return true;
+}
+
 struct JSCountHeapNode {
     void                *thing;
     JSGCTraceKind       kind;
@@ -555,16 +655,11 @@ static const struct TraceKindPair {
     { "all",        -1                  },
     { "object",     JSTRACE_OBJECT      },
     { "string",     JSTRACE_STRING      },
-#if JS_HAS_XML_SUPPORT
-    { "xml",        JSTRACE_XML         },
-#endif
 };
 
 static JSBool
 CountHeap(JSContext *cx, unsigned argc, jsval *vp)
 {
-    void* startThing;
-    JSGCTraceKind startTraceKind;
     jsval v;
     int32_t traceKind;
     JSString *str;
@@ -572,13 +667,11 @@ CountHeap(JSContext *cx, unsigned argc, jsval *vp)
     JSCountHeapNode *node;
     size_t counter;
 
-    startThing = NULL;
-    startTraceKind = JSTRACE_OBJECT;
+    Value startValue = UndefinedValue();
     if (argc > 0) {
         v = JS_ARGV(cx, vp)[0];
         if (JSVAL_IS_TRACEABLE(v)) {
-            startThing = JSVAL_TO_TRACEABLE(v);
-            startTraceKind = JSVAL_TRACE_KIND(v);
+            startValue = v;
         } else if (!JSVAL_IS_NULL(v)) {
             JS_ReportError(cx,
                            "the first argument is not null or a heap-allocated "
@@ -618,11 +711,10 @@ CountHeap(JSContext *cx, unsigned argc, jsval *vp)
     countTracer.traceList = NULL;
     countTracer.recycleList = NULL;
 
-    if (!startThing) {
+    if (startValue.isUndefined()) {
         JS_TraceRuntime(&countTracer.base);
     } else {
-        JS_SET_TRACING_NAME(&countTracer.base, "root");
-        JS_CallTracer(&countTracer.base, startThing, startTraceKind);
+        JS_CallValueTracer(&countTracer.base, startValue, "root");
     }
 
     counter = 0;
@@ -698,7 +790,7 @@ DumpHeapComplete(JSContext *cx, unsigned argc, jsval *vp)
         Value v = JS_ARGV(cx, vp)[0];
         if (v.isString()) {
             JSString *str = v.toString();
-            if (!fileNameBytes.encode(cx, str))
+            if (!fileNameBytes.encodeLatin1(cx, str))
                 return false;
             fileName = fileNameBytes.ptr();
         }
@@ -739,6 +831,13 @@ MJitChunkLimit(JSContext *cx, unsigned argc, jsval *vp)
         return JS_FALSE;
     }
 
+    for (CompartmentsIter c(cx->runtime); !c.done(); c.next()) {
+        if (c->lastAnimationTime != 0) {
+            JS_ReportError(cx, "Can't change chunk limit if code may be preserved");
+            return JS_FALSE;
+        }
+    }
+
     double t;
     if (!JS_ValueToNumber(cx, args[0], &t))
         return JS_FALSE;
@@ -783,6 +882,14 @@ EnableSPSProfilingAssertions(JSContext *cx, unsigned argc, jsval *vp)
     return true;
 }
 
+static JSBool
+DisableSPSProfiling(JSContext *cx, unsigned argc, jsval *vp)
+{
+    if (cx->runtime->spsProfiler.installed())
+        cx->runtime->spsProfiler.enable(false);
+    return true;
+}
+
 static JSBool
 DisplayName(JSContext *cx, unsigned argc, jsval *vp)
 {
@@ -799,6 +906,26 @@ DisplayName(JSContext *cx, unsigned argc, jsval *vp)
     return true;
 }
 
+JSBool
+js::testingFunc_inParallelSection(JSContext *cx, unsigned argc, jsval *vp)
+{
+    // If we were actually *in* a parallel section, then this function
+    // would be inlined to TRUE in ion-generated code.
+    JS_ASSERT(!InParallelSection());
+    JS_SET_RVAL(cx, vp, JSVAL_FALSE);
+    return true;
+}
+
+#ifndef JS_ION
+JSBool
+js::IsAsmJSCompilationAvailable(JSContext *cx, unsigned argc, Value *vp)
+{
+    CallArgs args = CallArgsFromVp(argc, vp);
+    args.rval().set(BooleanValue(false));
+    return true;
+}
+#endif
+
 static JSFunctionSpecWithHelp TestingFunctions[] = {
     JS_FN_HELP("gc", ::GC, 0, 0,
 "gc([obj] | 'compartment')",
@@ -820,8 +947,8 @@ static JSFunctionSpecWithHelp TestingFunctions[] = {
 "countHeap([start[, kind]])",
 "  Count the number of live GC things in the heap or things reachable from\n"
 "  start when it is given and is not null. kind is either 'all' (default) to\n"
-"  count all things or one of 'object', 'double', 'string', 'function',\n"
-"  'qname', 'namespace', 'xml' to count only things of that kind."),
+"  count all things or one of 'object', 'double', 'string', 'function'\n"
+"  to count only things of that kind."),
 
     JS_FN_HELP("makeFinalizeObserver", MakeFinalizeObserver, 0, 0,
 "makeFinalizeObserver()",
@@ -833,6 +960,10 @@ static JSFunctionSpecWithHelp TestingFunctions[] = {
 "  Return the current value of the finalization counter that is incremented\n"
 "  each time an object returned by the makeFinalizeObserver is finalized."),
 
+    JS_FN_HELP("gcPreserveCode", GCPreserveCode, 0, 0,
+"gcPreserveCode()",
+"  Preserve JIT code during garbage collections."),
+
 #ifdef JS_GC_ZEAL
     JS_FN_HELP("gczeal", GCZeal, 2, 0,
 "gczeal(level, [period])",
@@ -843,8 +974,8 @@ static JSFunctionSpecWithHelp TestingFunctions[] = {
 "    3: Collect when the window paints (browser only)\n"
 "    4: Verify pre write barriers between instructions\n"
 "    5: Verify pre write barriers between paints\n"
-"    6: Verify stack rooting (ignoring XML and Reflect)\n"
-"    7: Verify stack rooting (all roots)\n"
+"    6: Verify stack rooting\n"
+"    7: Verify stack rooting (yes, it's the same as 6)\n"
 "    8: Incremental GC in two slices: 1) mark roots 2) finish collection\n"
 "    9: Incremental GC in two slices: 1) mark all 2) new marking and finish\n"
 "   10: Incremental GC in multiple slices\n"
@@ -870,23 +1001,31 @@ static JSFunctionSpecWithHelp TestingFunctions[] = {
 "verifypostbarriers()",
 "  Start or end a run of the post-write barrier verifier."),
 
-    JS_FN_HELP("gcslice", GCSlice, 1, 0,
-"gcslice(n)",
-"  Run an incremental GC slice that marks about n objects."),
-
-    JS_FN_HELP("gcPreserveCode", GCPreserveCode, 0, 0,
-"gcPreserveCode()",
-"  Preserve JIT code during garbage collections."),
+    JS_FN_HELP("gcstate", GCState, 0, 0,
+"gcstate()",
+"  Report the global GC state."),
 
     JS_FN_HELP("deterministicgc", DeterministicGC, 1, 0,
 "deterministicgc(true|false)",
 "  If true, only allow determinstic GCs to run."),
 #endif
 
+    JS_FN_HELP("gcslice", GCSlice, 1, 0,
+"gcslice(n)",
+"  Run an incremental GC slice that marks about n objects."),
+
     JS_FN_HELP("validategc", ValidateGC, 1, 0,
 "validategc(true|false)",
 "  If true, a separate validation step is performed after an incremental GC."),
 
+    JS_FN_HELP("fullcompartmentchecks", FullCompartmentChecks, 1, 0,
+"fullcompartmentchecks(true|false)",
+"  If true, check for compartment mismatches before every GC."),
+
+    JS_FN_HELP("nondeterministicGetWeakMapKeys", NondeterminsticGetWeakMapKeys, 1, 0,
+"nondeterministicGetWeakMapKeys(weakmap)",
+"  Return an array of the keys in the given WeakMap."),
+
     JS_FN_HELP("internalConst", InternalConst, 1, 0,
 "internalConst(name)",
 "  Query an internal constant for the engine. See InternalConst source for\n"
@@ -916,21 +1055,30 @@ static JSFunctionSpecWithHelp TestingFunctions[] = {
 "  When 'slow' is false, then instrumentation is enabled, but the slow\n"
 "  assertions are disabled."),
 
+    JS_FN_HELP("disableSPSProfiling", DisableSPSProfiling, 1, 0,
+"disableSPSProfiling()",
+"  Disables SPS instrumentation"),
+
     JS_FN_HELP("displayName", DisplayName, 1, 0,
 "displayName(fn)",
 "  Gets the display name for a function, which can possibly be a guessed or\n"
 "  inferred name based on where the function was defined. This can be\n"
 "  different from the 'name' property on the function."),
 
+    JS_FN_HELP("isAsmJSCompilationAvailable", IsAsmJSCompilationAvailable, 0, 0,
+"isAsmJSCompilationAvailable",
+"  Returns whether asm.js compilation is currently available or whether it is disabled\n"
+"  (e.g., by the debugger)."),
+
+    JS_FN_HELP("inParallelSection", testingFunc_inParallelSection, 0, 0,
+"inParallelSection()",
+"  True if this code is executing within a parallel section."),
+
     JS_FS_HELP_END
 };
 
-namespace js {
-
 bool
-DefineTestingFunctions(JSContext *cx, HandleObject obj)
+js::DefineTestingFunctions(JSContext *cx, HandleObject obj)
 {
     return JS_DefineFunctionsWithHelp(cx, obj, TestingFunctions);
 }
-
-} /* namespace js */
