@@ -1,41 +1,6 @@
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is the Cisco Systems SIP Stack.
- *
- * The Initial Developer of the Original Code is
- * Cisco Systems (CSCO).
- * Portions created by the Initial Developer are Copyright (C) 2002
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *  Enda Mannion <emannion@cisco.com>
- *  Suhas Nandakumar <snandaku@cisco.com>
- *  Ethan Hugg <ehugg@cisco.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "cpr_types.h"
 #include "cpr_memory.h"
@@ -172,7 +137,7 @@ kpml_create_sm_key (kpml_key_t *key_p, line_t line, callid_t call_id,
 {
     static const char fname[] = "kpml_create_sm_key";
 
-    KPML_DEBUG(DEB_L_C_F_PREFIX" timer=0x%0x\n", 
+    KPML_DEBUG(DEB_L_C_F_PREFIX" timer=0x%0x\n",
 			   DEB_L_C_F_PREFIX_ARGS(KPML_INFO, line, call_id, fname), tmr_ptr);
 
     key_p->line = line;
@@ -210,14 +175,14 @@ kpml_match_line_call_id (kpml_data_t * kpml_data_p, kpml_key_t * key_p)
 /*
  *  Function: kpml_data_present_for_subid()
  *
- *  Parameters: sub_id_t - sub id sent int he message 
+ *  Parameters: sub_id_t - sub id sent int he message
  *
  *  Description: Callback function provided to link list
- *              to see if there kpml_data that matches a sub_id  
+ *              to see if there kpml_data that matches a sub_id
  *
  *  Returns: kpml_data_t * - Return kpml_data if a match is found
  */
-static kpml_data_t * 
+static kpml_data_t *
 kpml_data_for_subid(sub_id_t sub_id)
 {
     kpml_data_t *kpml_data;
@@ -283,18 +248,18 @@ kpml_stop_timer (void *timer_ptr)
 /*
  *  Function: kpml_clear_timers()
  *
- *  Parameters: kpml_data - pointer to kpml data 
+ *  Parameters: kpml_data - pointer to kpml data
  *
- *  Description: stop and delete all kpml timers 
+ *  Description: stop and delete all kpml timers
  *
- *  Returns: None 
+ *  Returns: None
  */
 static void
 kpml_clear_timers (kpml_data_t *kpml_data)
 {
     static const char fname[] = "kpml_clear_timers";
 
-    KPML_DEBUG(DEB_L_C_F_PREFIX"Release kpml timers.\n", 
+    KPML_DEBUG(DEB_L_C_F_PREFIX"Release kpml timers.\n",
                 DEB_L_C_F_PREFIX_ARGS(KPML_INFO, kpml_data->line, kpml_data->call_id, fname));
 
     kpml_stop_timer(kpml_data->inter_digit_timer);
@@ -328,15 +293,15 @@ kpml_start_timers (kpml_data_t *kpml_data)
 
     kpml_data->inter_digit_timer =
         cprCreateTimer("Interdigit timer", GSM_KPML_INTER_DIGIT_TIMER,
-                       TIMER_EXPIRATION, gsm_msg_queue);
+                       TIMER_EXPIRATION, gsm_msgq);
 
     kpml_data->critical_timer =
         cprCreateTimer("Criticaldigit timer", GSM_KPML_CRITICAL_DIGIT_TIMER,
-                       TIMER_EXPIRATION, gsm_msg_queue);
+                       TIMER_EXPIRATION, gsm_msgq);
 
     kpml_data->extra_digit_timer =
         cprCreateTimer("Extradigit timer", GSM_KPML_EXTRA_DIGIT_TIMER,
-                       TIMER_EXPIRATION, gsm_msg_queue);
+                       TIMER_EXPIRATION, gsm_msgq);
 
     /* Check if any of the timer cannot be allocated */
     if (kpml_data->inter_digit_timer == NULL ||
@@ -344,7 +309,7 @@ kpml_start_timers (kpml_data_t *kpml_data)
         kpml_data->extra_digit_timer == NULL) {
 
         /* generate error to indicate timer cannot be allocated */
-        KPML_ERROR(KPML_F_PREFIX"No memory to allocate timer\n", 
+        KPML_ERROR(KPML_F_PREFIX"No memory to allocate timer\n",
                     fname);
         return;
     }
@@ -381,7 +346,7 @@ kpml_restart_timers (kpml_data_t * kpml_data)
 {
     static const char fname[] = "kpml_restart_timers";
 
-    KPML_DEBUG(DEB_L_C_F_PREFIX"Restart all timers\n", 
+    KPML_DEBUG(DEB_L_C_F_PREFIX"Restart all timers\n",
                DEB_L_C_F_PREFIX_ARGS(KPML_INFO, kpml_data->line, kpml_data->call_id, fname));
 
     kpml_stop_timer(kpml_data->critical_timer);
@@ -633,11 +598,11 @@ kpml_inter_digit_timer_callback (void *kpml_key_p)
 kpml_data_t *kpml_get_kpml_data_from_kpml_id(uint32_t kpml_id)
 {
     kpml_data_t *kpml_data;
-	
+
     kpml_data = (kpml_data_t *) sll_next(s_kpml_list, NULL);
-	
+
     while (kpml_data != NULL && kpml_data->kpml_id != kpml_id) {
-		
+
         kpml_data = (kpml_data_t *) sll_next(s_kpml_list, kpml_data);
     }
     return(kpml_data);
@@ -695,12 +660,12 @@ kpml_start_subscription_timer (kpml_data_t * kpml_data, unsigned long duration)
 {
     static const char fname[] = "kpml_start_subscription_timer";
 
-    KPML_DEBUG(DEB_L_C_F_PREFIX"duration=%u\n", 
+    KPML_DEBUG(DEB_L_C_F_PREFIX"duration=%u\n",
                DEB_L_C_F_PREFIX_ARGS(KPML_INFO, kpml_data->line, kpml_data->call_id, fname), duration);
 
     kpml_data->sub_timer = cprCreateTimer("sub timer",
                                           GSM_KPML_SUBSCRIPTION_TIMER,
-                                          TIMER_EXPIRATION, gsm_msg_queue);
+                                          TIMER_EXPIRATION, gsm_msgq);
 
     kpml_data->sub_duration = duration;
 
@@ -903,7 +868,7 @@ boolean kpml_is_subscribed (callid_t call_id, line_t line)
         kpml_next_data = (kpml_data_t *) sll_next(s_kpml_list, kpml_data);
         if (kpml_data->pending_sub &&
             kpml_data->line == line &&
-            kpml_data->call_id == call_id) {   
+            kpml_data->call_id == call_id) {
             return TRUE;
         }
         kpml_data = kpml_next_data;
@@ -959,7 +924,7 @@ kpml_update_dialed_digits (line_t line, callid_t call_id, char digit)
             /* update the digit string */
             dial_len = strlen(kpml_data->kpmlDialed);
             if (dial_len >= MAX_DIALSTRING-1)
-            {  // not enough room 
+            {  // not enough room
                 KPML_ERROR(DEB_L_C_F_PREFIX"dial_len = [%d] too large\n", DEB_L_C_F_PREFIX_ARGS(KPML_INFO, line, call_id, fname), dial_len);
                 return (state);
             }
@@ -975,7 +940,7 @@ kpml_update_dialed_digits (line_t line, callid_t call_id, char digit)
                 kpml_data = kpml_next_data;
                 continue;
             }
-               
+
             if (digit == 0x0F) {
 
                 kpml_data->kpmlDialed[dial_len] = '#';
@@ -1218,7 +1183,7 @@ check_kpml_config (line_t line, callid_t call_id)
     lsm_state = lsm_get_state(call_id);
 
     if (lsm_state == LSM_S_NONE) {
-        KPML_ERROR(KPML_L_C_F_PREFIX"NO call\n", 
+        KPML_ERROR(KPML_L_C_F_PREFIX"NO call\n",
                     line, call_id, fname);
         return (KPML_BAD_EVENT);
     }
@@ -1310,8 +1275,8 @@ kpml_treat_regex (kpml_data_t *kpml_data)
 
     kpml_data->enable_backspace = FALSE;
 
-    KPML_DEBUG(DEB_L_C_F_PREFIX"regex=%u\n", 
-               DEB_L_C_F_PREFIX_ARGS(KPML_INFO, kpml_data->line, kpml_data->call_id, fname), 
+    KPML_DEBUG(DEB_L_C_F_PREFIX"regex=%u\n",
+               DEB_L_C_F_PREFIX_ARGS(KPML_INFO, kpml_data->line, kpml_data->call_id, fname),
                kpml_data->regex[indx].regexData);
 
     /* skip white space and check for backspace */
@@ -1412,7 +1377,7 @@ kpml_update_data (kpml_data_t *kpml_data, KPMLRequest *kpml_sub_data)
     KPML_DEBUG(DEB_L_C_F_PREFIX"regex=%u"
                "persistent=%d int-timer=%u critic-timer=%u, extra-timer=%u"
                "flush=%d longhold=%d longrepeat=%d nopartial=%d\n",
-               DEB_L_C_F_PREFIX_ARGS(KPML_INFO, kpml_data->line, kpml_data->call_id, fname), 
+               DEB_L_C_F_PREFIX_ARGS(KPML_INFO, kpml_data->line, kpml_data->call_id, fname),
 			   kpml_data->regex, kpml_data->persistent, kpml_data->inttimeout,
                kpml_data->crittimeout, kpml_data->extratimeout,
                kpml_data->flush, kpml_data->longhold,
@@ -1451,11 +1416,11 @@ kpml_terminate_subscription (ccsip_sub_not_data_t *msg)
         return;
     }
 
-    KPML_DEBUG(DEB_L_C_F_PREFIX"sub_id=%d, reason=%d\n", 
-                DEB_L_C_F_PREFIX_ARGS(KPML_INFO, msg->line_id, msg->gsm_id, fname), 
+    KPML_DEBUG(DEB_L_C_F_PREFIX"sub_id=%d, reason=%d\n",
+                DEB_L_C_F_PREFIX_ARGS(KPML_INFO, msg->line_id, msg->gsm_id, fname),
                 msg->sub_id, msg->reason_code);
     /*
-     * If the terminate reason is caused by local action, 
+     * If the terminate reason is caused by local action,
      * then there is no need to send any notify to the network.
      */
     switch (msg->reason_code) {
@@ -1465,8 +1430,8 @@ kpml_terminate_subscription (ccsip_sub_not_data_t *msg)
         /*
          * These errors are caused by failure or system being shutting down.
          * Subscription manager will automatically clean up the subscription
-         * The application just needs to clean up the associated data 
-         * strutures. 
+         * The application just needs to clean up the associated data
+         * strutures.
          */
         normal_terminate = FALSE;
         break;
@@ -1543,14 +1508,14 @@ kpml_receive_subscribe (ccsip_sub_not_data_t *msg)
     boolean is_empty_resubscribe = FALSE;
 
     if (kpml_get_config_value() == KPML_NONE) {
-        KPML_DEBUG(DEB_L_C_F_PREFIX"KPML disabled in config.\n", 
+        KPML_DEBUG(DEB_L_C_F_PREFIX"KPML disabled in config.\n",
                    DEB_L_C_F_PREFIX_ARGS(KPML_INFO, msg->line_id, msg->gsm_id, fname));
         return;
     }
 
     if (msg->line_id == 0 || msg->gsm_id == 0) {
 
-        KPML_ERROR(KPML_L_C_F_PREFIX"Line or call_id not correct\n", 
+        KPML_ERROR(KPML_L_C_F_PREFIX"Line or call_id not correct\n",
                     msg->line_id, msg->gsm_id, fname);
         (void) sub_int_subscribe_ack(CC_SRC_GSM, CC_SRC_SIP, msg->sub_id,
                                      KPML_BAD_EVENT, msg->sub_duration);
@@ -1586,8 +1551,8 @@ kpml_receive_subscribe (ccsip_sub_not_data_t *msg)
             if (kpml_data->sub_id != msg->sub_id) {
 
                 KPML_DEBUG(DEB_L_C_F_PREFIX"Terminate previous subscription \
-                           sub_id = %x\n", 
-						   DEB_L_C_F_PREFIX_ARGS(KPML_INFO, kpml_data->line, kpml_data->call_id, fname), 
+                           sub_id = %x\n",
+						   DEB_L_C_F_PREFIX_ARGS(KPML_INFO, kpml_data->line, kpml_data->call_id, fname),
 						   kpml_data->sub_id);
 
                 kpml_generate_notify(kpml_data, FALSE,
@@ -1598,11 +1563,11 @@ kpml_receive_subscribe (ccsip_sub_not_data_t *msg)
                                               msg->request_id, msg->event);
             }
 
-            KPML_DEBUG(DEB_L_C_F_PREFIX"Refresh Subscription\n", 
+            KPML_DEBUG(DEB_L_C_F_PREFIX"Refresh Subscription\n",
                        DEB_L_C_F_PREFIX_ARGS(KPML_INFO, kpml_data->line, kpml_data->call_id, fname));
             /* Refresh subscription without kpml body. This is CCM current behavior
              * which seems not to follow rfc4730. We support this for compatibility
-             * purpose, and intepret the body to be exactly the same as previous 
+             * purpose, and intepret the body to be exactly the same as previous
              * subscription. Only new subscription time changes.
              */
             if (kpml_sub_data == NULL) {
@@ -1619,7 +1584,7 @@ kpml_receive_subscribe (ccsip_sub_not_data_t *msg)
 
             kpml_data = kpml_update_data(kpml_data, kpml_sub_data);
 
-            KPML_DEBUG(DEB_L_C_F_PREFIX"Activate Subscription\n", 
+            KPML_DEBUG(DEB_L_C_F_PREFIX"Activate Subscription\n",
                        DEB_L_C_F_PREFIX_ARGS(KPML_INFO, kpml_data->line, kpml_data->call_id, fname));
         }
 
@@ -1750,12 +1715,12 @@ kpml_receive_subscribe (ccsip_sub_not_data_t *msg)
 
         lsm_state = lsm_get_state(kpml_data->call_id);
 
-        /* When GSM receives FEATURE event transitions its state. To avoid 
-         * transition during DTMF phase do not send out the event if the 
+        /* When GSM receives FEATURE event transitions its state. To avoid
+         * transition during DTMF phase do not send out the event if the
          * lsm state is > RINGOUT state. GSM has to know about the subscription
-         * only to extend the collect info (KPML_COLLECT_INFO) 
+         * only to extend the collect info (KPML_COLLECT_INFO)
          */
-         
+
         if ((lsm_state != LSM_S_NONE) && (lsm_state < LSM_S_RINGOUT)) {
 
             cc_feature(CC_SRC_GSM, kpml_data->call_id, kpml_data->line,
@@ -1792,7 +1757,7 @@ kpml_generate_subscribe_response (kpml_data_t * kpml_data, int resp_code)
 {
     static const char fname[] = "kpml_generate_subscribe_response";
 
-    KPML_DEBUG(DEB_L_C_F_PREFIX"SUB response\n", 
+    KPML_DEBUG(DEB_L_C_F_PREFIX"SUB response\n",
 		       DEB_L_C_F_PREFIX_ARGS(KPML_INFO, kpml_data->line, kpml_data->call_id, fname));
 
     (void) sub_int_subscribe_ack(CC_SRC_GSM, CC_SRC_SIP, kpml_data->sub_id,
@@ -1816,7 +1781,7 @@ kpml_receive_notify_response (ccsip_sub_not_data_t *msg)
     kpml_data_t *kpml_data;
     kpml_key_t kpml_key;
 
-    KPML_DEBUG(DEB_L_C_F_PREFIX"Notify response\n", 
+    KPML_DEBUG(DEB_L_C_F_PREFIX"Notify response\n",
                DEB_L_C_F_PREFIX_ARGS(KPML_INFO, msg->line_id, msg->gsm_id, fname));
 
     kpml_create_sm_key(&kpml_key, (line_t) msg->line_id, (callid_t) msg->gsm_id,
@@ -1872,7 +1837,7 @@ kpml_generate_notify (kpml_data_t *kpml_data, boolean no_body,
     char resp_str[10];
     ccsip_event_data_t *peventData = NULL;
 
-    DEF_DEBUG(DEB_L_C_F_PREFIX"RESP %u: \n", 
+    DEF_DEBUG(DEB_L_C_F_PREFIX"RESP %u: \n",
         DEB_L_C_F_PREFIX_ARGS(KPML_INFO, kpml_data->line, kpml_data->call_id, fname), resp_code);
 
     if (no_body == FALSE) {
@@ -1881,7 +1846,7 @@ kpml_generate_notify (kpml_data_t *kpml_data, boolean no_body,
             cpr_malloc(sizeof(ccsip_event_data_t));
 
         if (peventData == NULL) {
-            KPML_ERROR(KPML_L_C_F_PREFIX"No memory for eventdata\n", 
+            KPML_ERROR(KPML_L_C_F_PREFIX"No memory for eventdata\n",
                 kpml_data->line, kpml_data->call_id, fname);
             return;
         }
@@ -2143,13 +2108,13 @@ kpml_shutdown (void)
 
     while (kpml_data != NULL) {
 
-        /* 
+        /*
          * Clean up, remove from the list and deallocate the kpml_data
          */
         (void) kpml_clear_data(kpml_data, KPML_ONE_SHOT);
 
-        /* 
-         * The kpml_data is already freed above, get the next one 
+        /*
+         * The kpml_data is already freed above, get the next one
          * from the list's head
          */
         kpml_data = (kpml_data_t *) sll_next(s_kpml_list, NULL);
