@@ -15,6 +15,9 @@ __defineSetter__("PluralForm", function (val) {
   return this.PluralForm = val;
 });
 
+XPCOMUtils.defineLazyModuleGetter(this, "SafeBrowsing",
+  "resource://gre/modules/SafeBrowsing.jsm");
+
 const REMOTESERVICE_CONTRACTID = "@mozilla.org/toolkit/remote-service;1";
 const XUL_NAMESPACE = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
 var gURLBar = null;
@@ -177,7 +180,7 @@ const gFormSubmitObserver = {
     var style = win.getComputedStyle(element, null);
     var scale = win.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                    .getInterface(Components.interfaces.nsIDOMWindowUtils)
-                   .screenPixelsPerCSSPixel;
+                   .fullZoom;
 
     var offset = style.direction == 'rtl' ? parseInt(style.paddingRight) +
                                             parseInt(style.borderRightWidth) :
@@ -460,24 +463,35 @@ function Startup()
   // set home button tooltip text
   updateHomeButtonTooltip();
 
-  // initialize observers and listeners
-  var xw = window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
+  var lc = window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                  .getInterface(Components.interfaces.nsIWebNavigation)
-                 .QueryInterface(Components.interfaces.nsIDocShellTreeItem)
-                 .treeOwner
-                 .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
-                 .getInterface(Components.interfaces.nsIXULWindow);
+                 .QueryInterface(Components.interfaces.nsILoadContext);
+  if (lc.usePrivateBrowsing) {
+    gPrivate = window;
+    document.documentElement.removeAttribute("windowtype");
+    var titlemodifier = document.documentElement.getAttribute("titlemodifier");
+    if (titlemodifier)
+      titlemodifier += " ";
+    titlemodifier += document.documentElement.getAttribute("titleprivate");
+    document.documentElement.setAttribute("titlemodifier", titlemodifier);
+    document.title = titlemodifier;
+  }
+
+  // initialize observers and listeners
+  var xw = lc.QueryInterface(Components.interfaces.nsIDocShellTreeItem)
+             .treeOwner
+             .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
+             .getInterface(Components.interfaces.nsIXULWindow);
   xw.XULBrowserWindow = window.XULBrowserWindow = new nsBrowserStatusHandler();
 
-  if (Services.prefs.getBoolPref("browser.doorhanger.enabled")) {
-    XPCOMUtils.defineLazyGetter(window, "PopupNotifications", function() {
-      var tmp = {};
-      Components.utils.import("resource://gre/modules/PopupNotifications.jsm", tmp);
-      return XULBrowserWindow.popupNotifications = new tmp.PopupNotifications(
-          getBrowser(),
-          document.getElementById("notification-popup"),
-          document.getElementById("notification-popup-box"));
-    });
+  if (!window.content.opener &&
+      Services.prefs.getBoolPref("browser.doorhanger.enabled")) {
+    var tmp = {};
+    Components.utils.import("resource://gre/modules/PopupNotifications.jsm", tmp);
+    window.PopupNotifications = new tmp.PopupNotifications(
+        getBrowser(),
+        document.getElementById("notification-popup"),
+        document.getElementById("notification-popup-box"));
     // Setting the popup notification attribute causes the XBL to bind
     // and call the constructor again, so we have to destroy it first.
     gBrowser.getNotificationBox().destroy();
@@ -591,7 +605,7 @@ function Startup()
     browser.userTypedValue = uriToLoad;
     if ("arguments" in window && window.arguments.length >= 3) {
       loadURI(uriToLoad, window.arguments[2], window.arguments[3] || null,
-              window.arguments[4] || false);
+              window.arguments[4] || false, window.arguments[5] || false);
     } else {
       loadURI(uriToLoad);
     }
@@ -610,14 +624,11 @@ function Startup()
   else
     setTimeout(WindowFocusTimerCallback, 0, content);
 
-  // Perform default browser checking (after window opens).
-  setTimeout(checkForDefaultBrowser, 0);
-
   // hook up browser access support
   window.browserDOMWindow = new nsBrowserAccess();
 
   // hook up remote support
-  if (REMOTESERVICE_CONTRACTID in Components.classes) {
+  if (!gPrivate && REMOTESERVICE_CONTRACTID in Components.classes) {
     var remoteService =
       Components.classes[REMOTESERVICE_CONTRACTID]
                 .getService(Components.interfaces.nsIRemoteService);
@@ -660,13 +671,18 @@ function Startup()
   gBrowser.mPanelContainer.addEventListener("PreviewBrowserTheme", LightWeightThemeWebInstaller, false, true);
   gBrowser.mPanelContainer.addEventListener("ResetBrowserThemePreview", LightWeightThemeWebInstaller, false, true);
 
-  DownloadTaskbarIntegration.onBrowserWindowLoad(window);
+  if (!gPrivate) {
+    DownloadTaskbarIntegration.onBrowserWindowLoad(window);
 
-  // initialize the sync UI
-  gSyncUI.init();
+    // initialize the sync UI
+    gSyncUI.init();
 
-  // initialize the session-restore service
-  setTimeout(InitSessionStoreCallback, 0);
+    // initialize the session-restore service
+    setTimeout(InitSessionStoreCallback, 0);
+  }
+
+  // Bug 778855 - Perf regression if we do this here. To be addressed in bug 779008.
+  setTimeout(function() { SafeBrowsing.init(); }, 2000);
 }
 
 function UpdateNavBar()
@@ -1181,6 +1197,8 @@ const BrowserSearch = {
    * the sidebar and selected, null otherwise.
    */
   get searchSidebar() {
+    if (sidebarObj.never_built)
+      return null;
     var panel = sidebarObj.panels.get_panel_from_id("urn:sidebar:panel:search");
     return panel && isElementVisible(panel.get_iframe()) &&
            panel.get_iframe()
@@ -1265,7 +1283,7 @@ function QualifySearchTerm()
 function BrowserOpenWindow()
 {
   //opens a window where users can select a web location to open
-  var params = { action: "0", url: "" };
+  var params = { action: gPrivate ? "4" : "0", url: "" };
   openDialog("chrome://communicator/content/openLocation.xul", "_blank", "chrome,modal,titlebar", params);
   var postData = { };
   var url = getShortcutOrURI(params.url, postData);
@@ -1283,6 +1301,9 @@ function BrowserOpenWindow()
     case "3": // new tab
       gBrowser.selectedTab = gBrowser.addTab(url, {allowThirdPartyFixup: true, postData: postData.value});
       break;
+    case "4": // private
+      openNewPrivateWith(params.url);
+      break;
   }
 }
 
@@ -1535,14 +1556,17 @@ function BrowserCloseWindow()
   window.close();
 }
 
-function loadURI(uri, referrer, postData, allowThirdPartyFixup)
+function loadURI(uri, referrer, postData, allowThirdPartyFixup, isUTF8)
 {
   try {
     var flags = nsIWebNavigation.LOAD_FLAGS_NONE;
     if (allowThirdPartyFixup) {
       flags = nsIWebNavigation.LOAD_FLAGS_ALLOW_THIRD_PARTY_FIXUP;
     }
-    else if (typeof postData == "number") {
+    if (isUTF8) {
+      flags |= nsIWebNavigation.LOAD_FLAGS_URI_IS_UTF8;
+    }
+    if (!flags && typeof postData == "number") {
       // Deal with legacy code that passes load flags in the third argument.
       flags = postData;
       postData = null;
@@ -1588,6 +1612,7 @@ function handleURLBarCommand(aUserAction, aTriggeringEvent)
     var browser = getBrowser();
     var postData = {};
     url = getShortcutOrURI(url, postData);
+    var isUTF8 = browser.userTypedValue === null;
     // Accept both Control and Meta (=Command) as New-Window-Modifiers
     if (aTriggeringEvent &&
         (('ctrlKey' in aTriggeringEvent && aTriggeringEvent.ctrlKey) ||
@@ -1598,7 +1623,11 @@ function handleURLBarCommand(aUserAction, aTriggeringEvent)
         // Reset url in the urlbar
         URLBarSetURI();
         // Open link in new tab
-        var t = browser.addTab(url, {allowThirdPartyFixup: true, postData: postData.value});
+        var t = browser.addTab(url, {
+                  postData: postData.value,
+                  allowThirdPartyFixup: true,
+                  isUTF8: isUTF8
+                });
 
         // Focus new tab unless shift is pressed
         if (!shiftPressed)
@@ -1606,7 +1635,7 @@ function handleURLBarCommand(aUserAction, aTriggeringEvent)
       } else {
         // Open a new window with the URL
         var newWin = openDialog(getBrowserURL(), "_blank", "all,dialog=no", url,
-            null, null, postData.value, true);
+            null, null, postData.value, true, isUTF8);
         // Reset url in the urlbar
         URLBarSetURI();
 
@@ -1636,7 +1665,7 @@ function handleURLBarCommand(aUserAction, aTriggeringEvent)
     } else {
       // No modifier was pressed, load the URL normally and
       // focus the content area
-      loadURI(url, null, postData.value, true);
+      loadURI(url, null, postData.value, true, isUTF8);
       content.focus();
     }
   }
@@ -2101,34 +2130,6 @@ function URLBarClickHandler(aEvent)
       gURLBar.select();
 }
 
-// This function gets the shell service and has it check its setting
-// This will do nothing on platforms without a shell service.
-function checkForDefaultBrowser()
-{
-  const NS_SHELLSERVICE_CID = "@mozilla.org/suite/shell-service;1";
-
-  if (NS_SHELLSERVICE_CID in Components.classes) try {
-    const nsIShellService = Components.interfaces.nsIShellService;
-    var shellService = Components.classes["@mozilla.org/suite/shell-service;1"]
-                                 .getService(nsIShellService);
-    var appTypes = shellService.shouldBeDefaultClientFor;
-
-    // show the default client dialog only if we should check for the default
-    // client and we aren't already the default for the stored app types in
-    // shell.checkDefaultApps
-    if (appTypes && shellService.shouldCheckDefaultClient &&
-        !shellService.isDefaultClient(true, appTypes)) {
-      window.openDialog("chrome://communicator/content/defaultClientDialog.xul",
-                        "DefaultClient",
-                        "modal,centerscreen,chrome,resizable=no"); 
-      // Force the sidebar to build since the windows
-      // integration dialog has come up.
-      SidebarRebuild();
-    }
-  } catch (e) {
-  }
-}
-
 function ShowAndSelectContentsOfURLBar()
 {
   if (!isElementVisible(gURLBar)) {
@@ -2340,7 +2341,7 @@ function WindowIsClosing()
   var numtabs = cn.length;
   var reallyClose = true;
 
-  if (!/Mac/.test(navigator.platform) && isClosingLastBrowser()) {
+  if (!gPrivate && !/Mac/.test(navigator.platform) && isClosingLastBrowser()) {
     let closingCanceled = Components.classes["@mozilla.org/supports-PRBool;1"]
                                     .createInstance(Components.interfaces.nsISupportsPRBool);
     Services.obs.notifyObservers(closingCanceled, "browser-lastwindow-close-requested", null);
@@ -2352,7 +2353,7 @@ function WindowIsClosing()
     return true;
   }
 
-  if (numtabs > 1) {
+  if (!gPrivate && numtabs > 1) {
     var shouldPrompt = Services.prefs.getBoolPref("browser.tabs.warnOnClose");
     if (shouldPrompt) {
       //default to true: if it were false, we wouldn't get this far
