@@ -45,7 +45,7 @@ nsContextMenu.prototype = {
     this.isTextSelected = this.isTextSelection();
     this.isContentSelected = this.isContentSelection();
 
-    this.initPopupURL();
+    this.initPopupPrincipal();
 
     // Initialize (disable/remove) menu items.
     this.initItems();
@@ -76,8 +76,9 @@ nsContextMenu.prototype = {
 
   initOpenItems: function() {
     var showOpen = this.onSaveableLink || (this.inDirList && this.onLink);
-    this.showItem("context-openlink", showOpen);
     this.showItem("context-openlinkintab", showOpen);
+    this.showItem("context-openlink", showOpen && !gPrivate);
+    this.showItem("context-openlinkinprivatewindow", showOpen);
     this.showItem("context-sep-open", showOpen);
   },
 
@@ -158,7 +159,7 @@ nsContextMenu.prototype = {
     this.showItem("context-sep-properties",
                   !(this.inDirList || this.isContentSelected || this.onTextInput ||
                     this.onCanvas || this.onVideo || this.onAudio));
-    // Set As Wallpaper depends on whether an image was clicked on,
+    // Set Desktop Background depends on whether an image was clicked on,
     // and requires the shell service.
     var canSetDesktopBackground = false;
     if ("@mozilla.org/suite/shell-service;1" in Components.classes) try {
@@ -168,15 +169,15 @@ nsContextMenu.prototype = {
                     .canSetDesktopBackground;
     } catch (e) {
     }
-    this.showItem("context-setWallpaper",
+    this.showItem("context-setDesktopBackground",
                   canSetDesktopBackground && (this.onLoadedImage || this.onStandaloneImage));
 
     this.showItem("context-sep-image",
                   this.onLoadedImage || this.onStandaloneImage);
 
     if (canSetDesktopBackground && this.onLoadedImage)
-      // Disable the Set As Wallpaper menu item if we're still trying to load the image
-      this.setItemAttr("context-setWallpaper", "disabled",
+      // Disable the Set Desktop Background menu item if we're still trying to load the image
+      this.setItemAttr("context-setDesktopBackground", "disabled",
                        (("complete" in this.target) && !this.target.complete) ? "true" : null);
 
     this.showItem("context-fitimage", this.onStandaloneImage &&
@@ -244,7 +245,7 @@ nsContextMenu.prototype = {
                   !(this.isContentSelected || this.onTextInput ||
                     this.onStandaloneImage || this.onVideo || this.onAudio));
     this.showItem("context-bookmarklink", this.onLink && !this.onMailtoLink);
-    this.showItem("context-searchselect", this.isTextSelected && !this.onTextInput);
+    this.showItem("context-searchselect", this.isTextSelected);
     this.showItem("context-keywordfield", this.onTextInput && this.onKeywordField);
     this.showItem("frame", this.inFrame);
     this.showItem("frame-sep", this.inFrame);
@@ -253,18 +254,18 @@ nsContextMenu.prototype = {
                      this.autoDownload ? "valueSave" : "valueSaveAs");
 
     var blocking = true;
-      if (this.popupURL)
+      if (this.popupPrincipal)
         try {
           const PM = Components.classes["@mozilla.org/PopupWindowManager;1"]
                      .getService(Components.interfaces.nsIPopupWindowManager);
-          blocking = PM.testPermission(this.popupURL) ==
-                     Components.interfaces.nsIPopupWindowManager.DENY_POPUP;
+          blocking = PM.testPermission(this.popupPrincipal) == PM.DENY_POPUP;
         } catch (e) {
+          Components.utils.reportError(e);
         }
 
-    this.showItem("popupwindow-reject", this.popupURL && !blocking);
-    this.showItem("popupwindow-allow", this.popupURL && blocking);
-    this.showItem("context-sep-popup", this.popupURL);
+    this.showItem("popupwindow-reject", this.popupPrincipal && !blocking);
+    this.showItem("popupwindow-allow", this.popupPrincipal && blocking);
+    this.showItem("context-sep-popup", this.popupPrincipal);
 
     // BiDi UI
     this.showItem("context-sep-bidi", gShowBiDi);
@@ -436,7 +437,7 @@ nsContextMenu.prototype = {
     this.inSyntheticDoc        = false;
     this.hasBGImage            = false;
     this.bgImageURL            = "";
-    this.popupURL              = null;
+    this.popupPrincipal        = null;
     this.autoDownload          = false;
     this.isTextSelected        = false;
     this.isContentSelected     = false;
@@ -445,7 +446,7 @@ nsContextMenu.prototype = {
     // Remember the node that was clicked.
     this.target = aNode;
 
-    if (aNode.namespaceURI == xulNS || this.isTargetAFormControl(aNode)) {
+    if (aNode.namespaceURI == xulNS) {
       this.shouldDisplay = false;
       return;
     }
@@ -516,7 +517,7 @@ nsContextMenu.prototype = {
       else if (this.target instanceof HTMLInputElement) {
         this.onTextInput = this.isTargetATextBox(this.target);
         // allow spellchecking UI on all writable text boxes except passwords
-        if (!this.target.readOnly && !this.target.disabled &&
+        if (this.onTextInput && !this.target.readOnly &&
             this.target.mozIsTextField(true)) {
           this.possibleSpellChecking = true;
           InlineSpellCheckerUI.init(this.target.QueryInterface(Components.interfaces.nsIDOMNSEditableElement).editor);
@@ -525,8 +526,8 @@ nsContextMenu.prototype = {
         this.onKeywordField = this.isTargetAKeywordField(this.target);
       }
       else if (this.target instanceof HTMLTextAreaElement) {
-        this.onTextInput = true;
-        if (!this.target.readOnly && !this.target.disabled) {
+        this.onTextInput = this.isTextBoxEnabled(this.target);
+        if (this.onTextInput && !this.target.readOnly) {
           this.possibleSpellChecking = true;
           InlineSpellCheckerUI.init(this.target.QueryInterface(Components.interfaces.nsIDOMNSEditableElement).editor);
           InlineSpellCheckerUI.initFromEvent(aRangeParent, aRangeOffset);
@@ -658,7 +659,7 @@ nsContextMenu.prototype = {
     }
   },
 
-  initPopupURL: function() {
+  initPopupPrincipal: function() {
     // quick check: if no opener, it can't be a popup
     if (!window.content.opener)
       return;
@@ -676,13 +677,11 @@ nsContextMenu.prototype = {
       if (xulwin.contextFlags &
           CI.nsIWindowCreator2.PARENT_IS_LOADING_OR_RUNNING_TIMEOUT) {
         // do the pref settings allow site-by-site popup management?
-        const PB = Components.classes["@mozilla.org/preferences-service;1"]
-                   .getService(CI.nsIPrefBranch);
-        show = !PB.getBoolPref("dom.disable_open_during_load");
+        show = !Services.prefs.getBoolPref("dom.disable_open_during_load");
       }
       if (show) {
-        // initialize popupURL
-        this.popupURL = Services.io.newURI(window.content.opener.location.href, null, null);
+        // initialize popupPrincipal
+        this.popupPrincipal = window.content.opener.document.nodePrincipal;
       }
     } catch(e) {
     }
@@ -717,18 +716,17 @@ nsContextMenu.prototype = {
 
   // Block popup windows
   rejectPopupWindows: function(aAndClose) {
-    const PM = Components.classes["@mozilla.org/PopupWindowManager;1"]
-               .getService(Components.interfaces.nsIPopupWindowManager);
-    PM.add(this.popupURL, false);
+    Services.perms.addFromPrincipal(this.popupPrincipal, "popup",
+                                    Services.perms.DENY_ACTION);
     if (aAndClose)
-      Services.obs.notifyObservers(window, "popup-perm-close", this.popupURL.spec);
+      Services.obs.notifyObservers(window, "popup-perm-close",
+                                   this.popupPrincipal.URI.spec);
   },
 
   // Unblock popup windows
   allowPopupWindows: function() {
-    const PM = Components.classes["@mozilla.org/PopupWindowManager;1"]
-               .getService(Components.interfaces.nsIPopupWindowManager);
-    PM.add(this.popupURL, true);
+    Services.perms.addFromPrincipal(this.popupPrincipal, "popup",
+                                    Services.perms.ALLOW_ACTION);
   },
 
   // Block/Unblock image from loading in the future.
@@ -740,12 +738,6 @@ nsContextMenu.prototype = {
     Services.perms.remove(uri.host, "image");
   },
 
-  // Open linked-to URL in a new window.
-  openLink: function() {
-    // Determine linked-to URL.
-    return openNewWindowWith(this.linkURL, this.target.ownerDocument);
-  },
-
   // Open linked-to URL in a new tab.
   openLinkInTab: function(aEvent) {
     // Determine linked-to URL.
@@ -753,6 +745,16 @@ nsContextMenu.prototype = {
                           aEvent);
   },
 
+  // Open linked-to URL in a new window.
+  openLinkInWindow: function() {
+    return openNewWindowWith(this.linkURL, this.target.ownerDocument);
+  },
+
+  // Open linked-to URL in a private window.
+  openLinkInPrivateWindow: function() {
+    return openNewPrivateWith(this.linkURL, this.target.ownerDocument);
+  },
+
   // Open frame in a new tab.
   openFrameInTab: function(aEvent) {
     // Determine linked-to URL.
@@ -864,7 +866,8 @@ nsContextMenu.prototype = {
     ctxDraw.drawImage(video, 0, 0);
     saveImageURL(canvas.toDataURL("image/jpeg", ""), name, "SaveImageTitle",
                                   true, true,
-                                  this.target.ownerDocument.documentURIObject);
+                                  this.target.ownerDocument.documentURIObject,
+                                  this.target.ownerDocument);
   },
 
   // Full screen video playback
@@ -888,28 +891,9 @@ nsContextMenu.prototype = {
       openUILinkIn(this.bgImageURL, where, null, null, doc.documentURIObject);
   },
 
-  setWallpaper: function() {
-    // Confirm since it's annoying if you hit this accidentally.
-    var navigatorBundle = document.getElementById("bundle_navigator");
-    var promptTitle = navigatorBundle.getString("wallpaperConfirmTitle");
-    var promptMsg = navigatorBundle.getString("wallpaperConfirmMsg");
-    var promptConfirmButton = navigatorBundle.getString("wallpaperConfirmButton");
-
-    if (Services.prompt.confirmEx(window, promptTitle, promptMsg,
-                                  (Services.prompt.BUTTON_TITLE_IS_STRING *
-                                   Services.prompt.BUTTON_POS_0) +
-                                  (Services.prompt.BUTTON_TITLE_CANCEL *
-                                   Services.prompt.BUTTON_POS_1),
-                                  promptConfirmButton, null, null, null,
-                                  {value: false}) != 0)
-      return;
-
-    const nsIShellService = Components.interfaces.nsIShellService;
-
-    Components.classes["@mozilla.org/suite/shell-service;1"]
-              .getService(nsIShellService)
-              .setDesktopBackground(this.target,
-                                    nsIShellService.BACKGROUND_STRETCH);
+  setDesktopBackground: function() {
+    openDialog("chrome://communicator/content/setDesktopBackground.xul",
+               "_blank", "chrome,modal,titlebar,centerscreen", this.target);
   },
 
   // Save URL of clicked-on frame.
@@ -974,7 +958,7 @@ nsContextMenu.prototype = {
         if (aStatusCode == NS_ERROR_SAVE_LINK_AS_TIMEOUT) {
           // Do it the old fashioned way, which will pick the best filename
           // it can without waiting.
-          saveURL(linkURL, linkText, dialogTitle, bypassCache, true, doc.documentURIObject);
+          saveURL(linkURL, linkText, dialogTitle, bypassCache, true, doc.documentURIObject, doc);
         }
         if (this.extListener)
           this.extListener.onStopRequest(aRequest, aContext, aStatusCode);
@@ -1025,6 +1009,9 @@ nsContextMenu.prototype = {
 
     channel.loadFlags |= flags;
 
+    if (channel instanceof Components.interfaces.nsIPrivateBrowsingChannel)
+      channel.setPrivate(gPrivate);
+
     if (channel instanceof Components.interfaces.nsIHttpChannel) {
       channel.referrer = doc.documentURIObject;
       if (channel instanceof Components.interfaces.nsIHttpChannelInternal)
@@ -1043,17 +1030,17 @@ nsContextMenu.prototype = {
 
   // Save URL of clicked-on image, video, or audio.
   saveMedia: function() {
+    var doc = this.target.ownerDocument;
     if (this.onCanvas)
       // Bypass cache, since it's a data: URL.
       saveImageURL(this.target.toDataURL(), "canvas.png", "SaveImageTitle",
-                   true, true, null);
+                   true, true, null, doc);
     else if (this.onImage)
       saveImageURL(this.mediaURL, null, "SaveImageTitle", false, true,
-                   this.target.ownerDocument.documentURIObject);
+                   doc.documentURIObject, doc);
     else if (this.onVideo || this.onAudio) {
       var dialogTitle = this.onVideo ? "SaveVideoTitle" : "SaveAudioTitle";
-      this.saveHelper(this.mediaURL, null, dialogTitle, false,
-                      this.target.ownerDocument);
+      this.saveHelper(this.mediaURL, null, dialogTitle, false, doc);
     }
   },
 
@@ -1267,6 +1254,7 @@ nsContextMenu.prototype = {
    */
   isTextSelection: function() {
     var searchSelectText = this.searchSelected(16);
+
     if (!searchSelectText)
       return false;
 
@@ -1285,11 +1273,11 @@ nsContextMenu.prototype = {
 
     // format "Search <engine> for <selection>" string to show in menu
     const bundle = document.getElementById("contentAreaCommandsBundle");
-    var menuLabel = bundle.getFormattedString("contextMenuSearchText",
+    var menuLabel = bundle.getFormattedString("searchSelected",
                                               [engineName, searchSelectText]);
     this.setItemAttr("context-searchselect", "label", menuLabel);
     this.setItemAttr("context-searchselect", "accesskey",
-                     bundle.getString("contextMenuSearchText.accesskey"));
+                     bundle.getString("searchSelected.accesskey"));
 
     return true;
   },
@@ -1298,6 +1286,16 @@ nsContextMenu.prototype = {
     var focusedWindow = document.commandDispatcher.focusedWindow;
     var searchStr = focusedWindow.getSelection();
     searchStr = searchStr.toString();
+
+    if (this.onTextInput) {
+      var fElem = this.target;
+      if ((fElem instanceof HTMLInputElement &&
+           fElem.mozIsTextField(true)) ||
+           fElem instanceof HTMLTextAreaElement) {
+        searchStr = fElem.value.substring(fElem.selectionStart, fElem.selectionEnd);
+      }
+    }
+
     // searching for more than 150 chars makes no sense
     if (!aCharlen)
       aCharlen = 150;
@@ -1336,23 +1334,18 @@ nsContextMenu.prototype = {
            "contextMenu.hasBGImage = " + this.hasBGImage + "\n";
   },
 
-  // Returns true if aNode is a form control (except text boxes and images).
-  // This is used to disable the context menu for form controls.
-  isTargetAFormControl: function(aNode) {
-    if (aNode instanceof HTMLInputElement)
-      return (!aNode.mozIsTextField(false) && aNode.type != "image");
-
-    return (aNode instanceof HTMLButtonElement) ||
-           (aNode instanceof HTMLSelectElement) ||
-           (aNode instanceof HTMLOptionElement) ||
-           (aNode instanceof HTMLOptGroupElement);
+  isTextBoxEnabled: function(aNode) {
+    return !aNode.ownerDocument.defaultView
+                 .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
+                 .getInterface(Components.interfaces.nsIDOMWindowUtils)
+                 .isNodeDisabledForEvents(aNode);
   },
 
   isTargetATextBox: function(aNode) {
     if (aNode instanceof HTMLInputElement)
-      return aNode.mozIsTextField(false);
+      return aNode.mozIsTextField(false) && this.isTextBoxEnabled(aNode);
 
-    return (aNode instanceof HTMLTextAreaElement);
+    return aNode instanceof HTMLTextAreaElement && this.isTextBoxEnabled(aNode);
   },
 
   isTargetAKeywordField: function(aNode) {
@@ -1406,14 +1399,11 @@ nsContextMenu.prototype = {
         media.setAttribute("controls", "true");
         break;
       case "showstats":
-        var event = document.createEvent("CustomEvent");
-        event.initCustomEvent("media-showStatistics", false, true, true);
-        media.dispatchEvent(event);
-        break;
       case "hidestats":
-        var event = document.createEvent("CustomEvent");
-        event.initCustomEvent("media-showStatistics", false, true, false);
-        media.dispatchEvent(event);
+        var win = media.ownerDocument.defaultView;
+        var showing = aCommand == "showstats";
+        media.dispatchEvent(new win.CustomEvent("media-showStatistics",
+          { bubbles: false, cancelable: true, detail: showing }));
         break;
     }
   },
