@@ -291,9 +291,8 @@ FeedWriter.prototype = {
 
     if (change) {
       this._contentSandbox.document = this._document;
-      codeStr += "var event = document.createEvent('Events'); " +
-                 "event.initEvent('CheckboxStateChange', true, true);" +
-                 "checkbox.dispatchEvent(event);";
+      codeStr += "checkbox.dispatchEvent(new Event('CheckboxStateChange',"+
+                 "  { bubbles: true, cancelable: true }));";
     }
 
     Components.utils.evalInSandbox(codeStr, this._contentSandbox);
@@ -643,20 +642,18 @@ FeedWriter.prototype = {
    * @returns The display name of the application represented by the file.
    */
   _getFileDisplayName: function getFileDisplayName(file) {
-#ifdef XP_WIN
-    if (file instanceof Components.interfaces.nsILocalFileWin) {
+    if ("nsILocalFileWin" in Components.interfaces &&
+        file instanceof Components.interfaces.nsILocalFileWin) {
       try {
         return file.getVersionInfoField("FileDescription");
       } catch (e) {}
     }
-#endif
-#ifdef XP_MACOSX
-    if (file instanceof Components.interfaces.nsILocalFileMac) {
+    else if ("nsILocalFileMac" in Components.interfaces &&
+             file instanceof Components.interfaces.nsILocalFileMac) {
       try {
         return file.bundleDisplayName;
       } catch (e) {}
     }
-#endif
     return file.leafName;
   },
 
@@ -715,18 +712,11 @@ FeedWriter.prototype = {
       if (fp.show() == Components.interfaces.nsIFilePicker.returnOK) {
         this._selectedApp = fp.file;
         if (this._selectedApp) {
-          // XXXben - we need to compare this with the running instance executable
-          //          just don't know how to do that via script...
-          // XXXmano TBD: can probably add this to nsIShellService
-#ifdef XP_WIN
-#expand           if (fp.file.leafName != "__MOZ_APP_NAME__.exe") {
-#else
-#ifdef XP_MACOSX
-#expand           if (fp.file.leafName != "__MOZ_MACBUNDLE_NAME__") {
-#else
-#expand           if (fp.file.leafName != "__MOZ_APP_NAME__-bin") {
-#endif
-#endif
+          var dirsvc = Components.classes["@mozilla.org/file/directory_service;1"]
+                                 .getService(Components.interfaces.nsIProperties);
+
+          var file = dirsvc.get("XREExeF", Components.interfaces.nsILocalFile);
+          if (fp.file.leafName != file.leafName) {
             this._initMenuItemWithFile(this._contentSandbox.selectedAppMenuItem,
                                        this._selectedApp);
 
@@ -798,17 +788,22 @@ FeedWriter.prototype = {
     }
   },
 
-  // nsIDomEventListener
+  // nsIDOMEventListener
   handleEvent: function(event) {
-    if (event.target.ownerDocument != this._document) {
+    if (event.target != this._document &&
+        event.target.ownerDocument != this._document) {
       LOG("FeedWriter.handleEvent: Someone passed the feed writer as a listener to the events of another document!");
       return;
     }
 
-    if (event.type == "command") {
+    if (event.type == "load")
+      this._writeContent();
+    else if (event.type == "unload")
+      this._close();
+    else if (event.type == "command") {
       switch (event.target.getAttribute("anonid")) {
         case "subscribeButton":
-          this.subscribe();
+          this._subscribe();
           break;
         case "chooseApplicationMenuItem":
           /* Bug 351263: Make sure to not steal focus if the "Choose
@@ -1095,8 +1090,8 @@ FeedWriter.prototype = {
   _feedPrincipal: null,
   _handlersMenuList: null,
 
-  // nsIFeedWriter
-  init: function init(aWindow) {
+  // nsIDOMGlobalObjectConstructor
+  constructor: function constructor(aWindow) {
     this._feedURI = this._getOriginalURI(aWindow);
     if (!this._feedURI)
       return;
@@ -1128,9 +1123,12 @@ FeedWriter.prototype = {
     prefs.addObserver(PREF_AUDIO_SELECTED_READER, this, false);
     prefs.addObserver(PREF_AUDIO_SELECTED_WEB, this, false);
     prefs.addObserver(PREF_AUDIO_SELECTED_APP, this, false);
+
+    this._window.addEventListener("load", this, false);
+    this._window.addEventListener("unload", this, false);
   },
 
-  writeContent: function writeContent() {
+  _writeContent: function writeContent() {
     if (!this._window)
       return;
 
@@ -1149,7 +1147,9 @@ FeedWriter.prototype = {
     }
   },
 
-  close: function close() {
+  _close: function close() {
+    this._window.removeEventListener("load", this, false);
+    this._window.removeEventListener("unload", this, false);
     this._getUIElement("handlersMenuPopup")
         .removeEventListener("command", this, false);
     this._getUIElement("subscribeButton")
@@ -1188,7 +1188,7 @@ FeedWriter.prototype = {
     }
   },
 
-  subscribe: function subscribe() {
+  _subscribe: function subscribe() {
     var feedType = this._getFeedType();
 
     // Subscribe to the feed using the selected handler and save prefs
@@ -1319,8 +1319,14 @@ FeedWriter.prototype = {
     }
     let faviconURI = makeURI(readerURI.resolve("/favicon.ico"));
     let self = this;
-    this._faviconService.setAndFetchFaviconForPage(readerURI, faviconURI, false,
-                                                   this._faviconService.FAVICON_LOAD_NON_PRIVATE,
+    var isPB = this._window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
+                   .getInterface(Components.interfaces.nsIWebNavigation)
+                   .QueryInterface(Components.interfaces.nsIDocShell)
+                   .QueryInterface(Components.interfaces.nsILoadContext)
+                   .usePrivateBrowsing;
+    var flags = isPB ? this._faviconService.FAVICON_LOAD_PRIVATE :
+                       this._faviconService.FAVICON_LOAD_NON_PRIVATE;
+    this._faviconService.setAndFetchFaviconForPage(readerURI, faviconURI, false, flags,
       function(aURI, aDataLen, aData, aMimeType) {
         if (aDataLen > 0) {
           let dataURL = "data:" + aMimeType + ";base64," +
@@ -1338,9 +1344,9 @@ FeedWriter.prototype = {
   classID: FEEDWRITER_CID,
   classInfo: XPCOMUtils.generateCI({classID: FEEDWRITER_CID,
                                     contractID: FEEDWRITER_CONTRACTID,
-                                    interfaces: [Components.interfaces.nsIFeedWriter],
+                                    interfaces: [],
                                     flags: Components.interfaces.nsIClassInfo.DOM_OBJECT}),
-  QueryInterface: XPCOMUtils.generateQI([Components.interfaces.nsIFeedWriter,
+  QueryInterface: XPCOMUtils.generateQI([ Components.interfaces.nsIDOMGlobalObjectConstructor,
                                           Components.interfaces.nsIDOMEventListener,
                                           Components.interfaces.nsINavHistoryObserver,
                                           Components.interfaces.nsIObserver])
